package gr.cite.annotation.integrationevent.inbox.annotationentitiesremoval;

import gr.cite.annotation.common.validation.BaseValidator;
import gr.cite.annotation.common.validation.UuidValidator;
import gr.cite.annotation.convention.ConventionService;
import gr.cite.annotation.errorcode.ErrorThesaurusProperties;
import gr.cite.annotation.integrationevent.TrackedEvent;
import gr.cite.tools.validation.ValidatorFactory;
import gr.cite.tools.validation.specification.Specification;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;
import java.util.UUID;

public class AnnotationEntitiesRemovalIntegrationEvent extends TrackedEvent {

    private List<UUID> entityIds;

    public static final String _entityIds = "entityIds";

    public List<UUID> getEntityIds() {
        return entityIds;
    }

    public void setEntityIds(List<UUID> entityIds) {
        this.entityIds = entityIds;
    }

    @Component(AnnotationEntitiesRemovalIntegrationEventValidator.ValidatorName)
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public static class AnnotationEntitiesRemovalIntegrationEventValidator extends BaseValidator<AnnotationEntitiesRemovalIntegrationEvent> {

        public static final String ValidatorName = "AnnotationEntitiesRemovalIntegrationEventValidator";

        private final MessageSource messageSource;

        private final ValidatorFactory validatorFactory;

        protected AnnotationEntitiesRemovalIntegrationEventValidator(ConventionService conventionService, ErrorThesaurusProperties errors, MessageSource messageSource, ValidatorFactory validatorFactory) {
            super(conventionService, errors);
            this.messageSource = messageSource;
            this.validatorFactory = validatorFactory;
        }

        @Override
        protected Class<AnnotationEntitiesRemovalIntegrationEvent> modelClass() {
            return AnnotationEntitiesRemovalIntegrationEvent.class;
        }

        @Override
        protected List<Specification> specifications(AnnotationEntitiesRemovalIntegrationEvent item) {
            return Arrays.asList(
                    this.spec()
                            .must(() -> !this.isListNullOrEmpty(item.getEntityIds()))
                            .failOn(AnnotationEntitiesRemovalIntegrationEvent._entityIds).failWith(messageSource.getMessage("Validation_Required", new Object[]{AnnotationEntitiesRemovalIntegrationEvent._entityIds}, LocaleContextHolder.getLocale())),
                    this.navSpec()
                            .iff(() -> !this.isListNullOrEmpty(item.getEntityIds()))
                            .on(AnnotationEntitiesRemovalIntegrationEvent._entityIds)
                            .over(item.getEntityIds())
                            .using((i) -> this.validatorFactory.validator(UuidValidator.class))
            );
        }
    }

}
