package gr.cite.annotation.integrationevent.inbox.annotationentitiestouch;

import gr.cite.annotation.common.validation.BaseValidator;
import gr.cite.annotation.common.validation.UuidValidator;
import gr.cite.annotation.convention.ConventionService;
import gr.cite.annotation.errorcode.ErrorThesaurusProperties;
import gr.cite.annotation.integrationevent.TrackedEvent;
import gr.cite.tools.validation.ValidatorFactory;
import gr.cite.tools.validation.specification.Specification;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;
import java.util.UUID;

public class AnnotationEntitiesTouchedIntegrationEvent extends TrackedEvent {

    private List<AnnotationEntityTouchedIntegrationEvent> events;

    public static final String _events = "events";

    public List<AnnotationEntityTouchedIntegrationEvent> getEvents() {
        return events;
    }

    public void setEvents(List<AnnotationEntityTouchedIntegrationEvent> events) {
        this.events = events;
    }

    @Component(AnnotationEntitiesTouchedIntegrationEventValidator.ValidatorName)
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public static class AnnotationEntitiesTouchedIntegrationEventValidator extends BaseValidator<AnnotationEntitiesTouchedIntegrationEvent> {

        public static final String ValidatorName = "AnnotationEntitiesTouchedIntegrationEventValidator";

        private final MessageSource messageSource;

        private final ValidatorFactory validatorFactory;

        protected AnnotationEntitiesTouchedIntegrationEventValidator(ConventionService conventionService, ErrorThesaurusProperties errors, MessageSource messageSource, ValidatorFactory validatorFactory) {
            super(conventionService, errors);
            this.messageSource = messageSource;
            this.validatorFactory = validatorFactory;
        }

        @Override
        protected Class<AnnotationEntitiesTouchedIntegrationEvent> modelClass() {
            return AnnotationEntitiesTouchedIntegrationEvent.class;
        }

        @Override
        protected List<Specification> specifications(AnnotationEntitiesTouchedIntegrationEvent item) {
            return Arrays.asList(
                    this.spec()
                            .must(() -> !this.isListNullOrEmpty(item.getEvents()))
                            .failOn(AnnotationEntitiesTouchedIntegrationEvent._events).failWith(messageSource.getMessage("Validation_Required", new Object[]{AnnotationEntitiesTouchedIntegrationEvent._events}, LocaleContextHolder.getLocale())),
                    this.navSpec()
                            .iff(() -> !this.isListNullOrEmpty(item.getEvents()))
                            .on(AnnotationEntitiesTouchedIntegrationEvent._events)
                            .over(item.getEvents())
                            .using((i) -> this.validatorFactory.validator(AnnotationEntityTouchedIntegrationEvent.AnnotationEntityTouchedIntegrationEventValidator.class))
            );
        }
    }

    public static class AnnotationEntityTouchedIntegrationEvent {

        private UUID entityId;

        public static final String _entityId = "entityId";

        private List<UUID> userIds;

        public static final String _userIds = "userIds";

        public UUID getEntityId() {
            return entityId;
        }

        public void setEntityId(UUID entityId) {
            this.entityId = entityId;
        }

        public List<UUID> getUserIds() {
            return userIds;
        }

        public void setUserIds(List<UUID> userIds) {
            this.userIds = userIds;
        }

        @Component(AnnotationEntityTouchedIntegrationEvent.AnnotationEntityTouchedIntegrationEventValidator.ValidatorName)
        @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
        public static class AnnotationEntityTouchedIntegrationEventValidator extends BaseValidator<AnnotationEntityTouchedIntegrationEvent> {

            public static final String ValidatorName = "AnnotationEntityTouchedIntegrationEventValidator";

            private final MessageSource messageSource;

            private final ValidatorFactory validatorFactory;

            protected AnnotationEntityTouchedIntegrationEventValidator(ConventionService conventionService, ErrorThesaurusProperties errors, MessageSource messageSource, ValidatorFactory validatorFactory) {
                super(conventionService, errors);
                this.messageSource = messageSource;
                this.validatorFactory = validatorFactory;
            }

            @Override
            protected Class<AnnotationEntityTouchedIntegrationEvent> modelClass() {
                return AnnotationEntityTouchedIntegrationEvent.class;
            }

            @Override
            protected List<Specification> specifications(AnnotationEntityTouchedIntegrationEvent item) {
                return Arrays.asList(
                        this.spec()
                                .iff(() -> !this.isNull(item.getEntityId()))
                                .must(() -> this.isValidGuid(item.getEntityId()))
                                .failOn(AnnotationEntityTouchedIntegrationEvent._entityId).failWith(messageSource.getMessage("Validation_Required", new Object[]{AnnotationEntityTouchedIntegrationEvent._entityId}, LocaleContextHolder.getLocale())),
                        this.spec()
                                .must(() -> !this.isListNullOrEmpty(item.getUserIds()))
                                .failOn(AnnotationEntityTouchedIntegrationEvent._userIds).failWith(messageSource.getMessage("Validation_Required", new Object[]{AnnotationEntityTouchedIntegrationEvent._userIds}, LocaleContextHolder.getLocale())),
                        this.navSpec()
                                .iff(() -> !this.isListNullOrEmpty(item.getUserIds()))
                                .on(AnnotationEntityTouchedIntegrationEvent._userIds)
                                .over(item.getUserIds())
                                .using((i) -> this.validatorFactory.validator(UuidValidator.class))
                );
            }
        }

    }

}
