package eu.eudat.elastic.elasticbuilder.nested;

import eu.eudat.commons.enums.IsActive;
import eu.eudat.convention.ConventionService;
import eu.eudat.data.DescriptionEntity;
import eu.eudat.data.DmpUserEntity;
import eu.eudat.data.UserEntity;
import eu.eudat.elastic.data.nested.NestedCollaboratorElasticEntity;
import eu.eudat.elastic.elasticbuilder.BaseElasticBuilder;
import eu.eudat.model.User;
import eu.eudat.query.DescriptionTemplateQuery;
import eu.eudat.query.UserQuery;
import gr.cite.tools.data.query.QueryFactory;
import gr.cite.tools.exception.MyApplicationException;
import gr.cite.tools.fieldset.BaseFieldSet;
import gr.cite.tools.logging.LoggerService;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.stream.Collectors;

@Component
@Scope(value = ConfigurableBeanFactory.SCOPE_PROTOTYPE)
public class NestedCollaboratorElasticBuilder extends BaseElasticBuilder<NestedCollaboratorElasticEntity, DmpUserEntity> {

    private final QueryFactory queryFactory;
    @Autowired
    public NestedCollaboratorElasticBuilder(
		    ConventionService conventionService, QueryFactory queryFactory) {
        super(conventionService, new LoggerService(LoggerFactory.getLogger(NestedCollaboratorElasticBuilder.class)));
	    this.queryFactory = queryFactory;
    }

    @Override
    public List<NestedCollaboratorElasticEntity> build(List<DmpUserEntity> data) throws MyApplicationException {
        if (data == null)
            return new ArrayList<>();

        List<NestedCollaboratorElasticEntity> models = new ArrayList<>();
        Map<UUID, String> userNameMap = this.queryFactory.query(UserQuery.class).isActive(IsActive.Active).ids(data.stream().map(DmpUserEntity::getUserId).distinct().collect(Collectors.toList())).collectAs(new BaseFieldSet().ensure(User._id).ensure(User._name))
                .stream().collect(Collectors.toMap(UserEntity::getId, UserEntity::getName));
        for (DmpUserEntity d : data) {
            NestedCollaboratorElasticEntity m = new NestedCollaboratorElasticEntity();
            m.setId(d.getId());
            m.setRole(d.getRole());
            m.setName(userNameMap.getOrDefault(d.getUserId(), d.getUserId().toString())); 
            models.add(m);
        }
        this.logger.debug("build {} items", Optional.of(models).map(List::size).orElse(0));
        return models;
    }
}
