package eu.eudat.integrationevent.outbox.notification;

import eu.eudat.commons.scope.tenant.TenantScope;
import eu.eudat.integrationevent.outbox.OutboxIntegrationEvent;
import eu.eudat.integrationevent.outbox.OutboxService;
import gr.cite.tools.logging.LoggerService;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.context.annotation.RequestScope;

import javax.management.InvalidApplicationException;
import java.util.UUID;

@Component
@RequestScope
public class NotifyIntegrationEventHandlerImpl implements NotifyIntegrationEventHandler {

    private static final LoggerService logger = new LoggerService(LoggerFactory.getLogger(NotifyIntegrationEventHandlerImpl.class));

    private final OutboxService outboxService;
    private final TenantScope tenantScope;

    @Autowired
    public NotifyIntegrationEventHandlerImpl(
		    OutboxService outboxService, TenantScope tenantScope) {
        this.outboxService = outboxService;
	    this.tenantScope = tenantScope;
    }

    @Override
    public void handle(NotifyIntegrationEvent event) throws InvalidApplicationException {
        OutboxIntegrationEvent message = new OutboxIntegrationEvent();
        message.setMessageId(UUID.randomUUID());
        message.setType(OutboxIntegrationEvent.NOTIFY);
        message.setEvent(event);
        if (this.tenantScope.isSet()) message.setTenantId(tenantScope.getTenant());
        this.outboxService.publish(message);
    }
}
