package eu.eudat.model.builder.actionconfirmation;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.types.actionconfirmation.DmpInvitationEntity;
import eu.eudat.convention.ConventionService;
import eu.eudat.model.actionconfirmation.DmpInvitation;
import eu.eudat.model.builder.BaseBuilder;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.exception.MyApplicationException;
import gr.cite.tools.fieldset.FieldSet;
import gr.cite.tools.logging.DataLogEntry;
import gr.cite.tools.logging.LoggerService;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.*;

@Component
@Scope(value = ConfigurableBeanFactory.SCOPE_PROTOTYPE)
public class DmpInvitationBuilder extends BaseBuilder<DmpInvitation, DmpInvitationEntity> {

    private EnumSet<AuthorizationFlags> authorize = EnumSet.of(AuthorizationFlags.None);

    @Autowired
    public DmpInvitationBuilder(
            ConventionService conventionService) {
        super(conventionService, new LoggerService(LoggerFactory.getLogger(DmpInvitationBuilder.class)));
    }

    public DmpInvitationBuilder authorize(EnumSet<AuthorizationFlags> values) {
        this.authorize = values;
        return this;
    }

    @Override
    public List<DmpInvitation> build(FieldSet fields, List<DmpInvitationEntity> data) throws MyApplicationException {
        this.logger.debug("building for {} items requesting {} fields", Optional.ofNullable(data).map(List::size).orElse(0), Optional.ofNullable(fields).map(FieldSet::getFields).map(Set::size).orElse(0));
        this.logger.trace(new DataLogEntry("requested fields", fields));
        if (fields == null || data == null || fields.isEmpty())
            return new ArrayList<>();


        List<DmpInvitation> models = new ArrayList<>();
        for (DmpInvitationEntity d : data) {
            DmpInvitation m = new DmpInvitation();
            if (fields.hasField(this.asIndexer(DmpInvitation._email))) m.setEmail(d.getEmail());
            if (fields.hasField(this.asIndexer(DmpInvitation._dmpId))) m.setDmpId(d.getDmpId());
            if (fields.hasField(this.asIndexer(DmpInvitation._role))) m.setRole(d.getRole());

            models.add(m);
        }
        this.logger.debug("build {} items", Optional.of(models).map(List::size).orElse(0));
        return models;
    }
}
