package eu.eudat.model.censorship;

import eu.eudat.authorization.Permission;
import eu.eudat.convention.ConventionService;
import eu.eudat.model.Dmp;
import eu.eudat.model.censorship.dmpproperties.DmpPropertiesCensor;
import gr.cite.commons.web.authz.service.AuthorizationService;
import gr.cite.tools.data.censor.CensorFactory;
import gr.cite.tools.fieldset.FieldSet;
import gr.cite.tools.logging.DataLogEntry;
import gr.cite.tools.logging.LoggerService;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.UUID;

@Component
@Scope(value = ConfigurableBeanFactory.SCOPE_PROTOTYPE)
public class DmpCensor extends BaseCensor {

    private static final LoggerService logger = new LoggerService(LoggerFactory.getLogger(DmpCensor.class));

    protected final AuthorizationService authService;

    protected final CensorFactory censorFactory;

    public DmpCensor(ConventionService conventionService,
                              AuthorizationService authService,
                              CensorFactory censorFactory) {
        super(conventionService);
        this.authService = authService;
        this.censorFactory = censorFactory;
    }

    public void censor(FieldSet fields, UUID userId) {
        logger.debug(new DataLogEntry("censoring fields", fields));
        if (fields == null || fields.isEmpty())
            return;

        this.authService.authorizeForce(Permission.BrowseDmp, Permission.DeferredAffiliation);

        FieldSet dmpUsersFields = fields.extractPrefixed(this.asIndexerPrefix(Dmp._dmpUsers));
        this.censorFactory.censor(DmpUserCensor.class).censor(dmpUsersFields, userId);
        FieldSet dmpReferencesFields = fields.extractPrefixed(this.asIndexerPrefix(Dmp._dmpReferences));
        this.censorFactory.censor(DmpReferenceCensor.class).censor(dmpReferencesFields, userId);
        FieldSet creatorFields = fields.extractPrefixed(this.asIndexerPrefix(Dmp._creator));
        this.censorFactory.censor(UserCensor.class).censor(creatorFields, userId);
        FieldSet doisFields = fields.extractPrefixed(this.asIndexerPrefix(Dmp._entityDois));
        this.censorFactory.censor(EntityDoiCensor.class).censor(doisFields, userId);
        FieldSet propertiesFields = fields.extractPrefixed(this.asIndexerPrefix(Dmp._properties));
        this.censorFactory.censor(DmpPropertiesCensor.class).censor(propertiesFields, userId);
    }
}
