package eu.eudat.model.censorship;

import eu.eudat.authorization.Permission;
import eu.eudat.convention.ConventionService;
import eu.eudat.model.DmpReference;
import eu.eudat.model.censorship.dmpreference.DmpReferenceDataCensor;
import gr.cite.commons.web.authz.service.AuthorizationService;
import gr.cite.tools.data.censor.CensorFactory;
import gr.cite.tools.fieldset.FieldSet;
import gr.cite.tools.logging.DataLogEntry;
import gr.cite.tools.logging.LoggerService;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.UUID;

@Component
@Scope(value = ConfigurableBeanFactory.SCOPE_PROTOTYPE)
public class DmpReferenceCensor extends BaseCensor {

    private static final LoggerService logger = new LoggerService(LoggerFactory.getLogger(DmpReferenceCensor.class));

    protected final AuthorizationService authService;
    protected final CensorFactory censorFactory;

    public DmpReferenceCensor(ConventionService conventionService,
                              AuthorizationService authService,
                              CensorFactory censorFactory) {
        super(conventionService);
        this.authService = authService;
        this.censorFactory = censorFactory;
    }

    public void censor(FieldSet fields, UUID userId) {
        logger.debug(new DataLogEntry("censoring fields", fields));
        if (fields == null || fields.isEmpty())
            return;

        this.authService.authorizeForce(Permission.BrowseDmpDescriptionTemplate, Permission.DeferredAffiliation);
        FieldSet dmpFields = fields.extractPrefixed(this.asIndexerPrefix(DmpReference._dmp));
        this.censorFactory.censor(DmpCensor.class).censor(dmpFields, userId);
        FieldSet templateFields = fields.extractPrefixed(this.asIndexerPrefix(DmpReference._reference));
        this.censorFactory.censor(ReferenceCensor.class).censor(templateFields, userId);
        FieldSet dataFields = fields.extractPrefixed(this.asIndexerPrefix(DmpReference._data));
        this.censorFactory.censor(DmpReferenceDataCensor.class).censor(dataFields, userId);
    }

}
