package eu.eudat.model.censorship;

import eu.eudat.authorization.Permission;
import eu.eudat.convention.ConventionService;
import eu.eudat.model.Reference;
import eu.eudat.model.censorship.referencedefinition.DefinitionCensor;
import gr.cite.commons.web.authz.service.AuthorizationService;
import gr.cite.tools.data.censor.CensorFactory;
import gr.cite.tools.fieldset.FieldSet;
import gr.cite.tools.logging.DataLogEntry;
import gr.cite.tools.logging.LoggerService;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.UUID;

@Component
@Scope(value = ConfigurableBeanFactory.SCOPE_PROTOTYPE)
public class ReferenceCensor extends BaseCensor {

    private static final LoggerService logger = new LoggerService(LoggerFactory.getLogger(ReferenceCensor.class));

    protected final AuthorizationService authService;
    protected final CensorFactory censorFactory;

    public ReferenceCensor(ConventionService conventionService,
                           AuthorizationService authService,
                           CensorFactory censorFactory) {
        super(conventionService);
        this.authService = authService;
        this.censorFactory = censorFactory;
    }

    public void censor(FieldSet fields, UUID userId) {
        logger.debug(new DataLogEntry("censoring fields", fields));
        if (fields == null || fields.isEmpty())
            return;

        this.authService.authorizeForce(Permission.BrowseReference, Permission.DeferredAffiliation);
        FieldSet definitionFields = fields.extractPrefixed(this.asIndexerPrefix(Reference._definition));
        this.censorFactory.censor(DefinitionCensor.class).censor(definitionFields, userId);
        FieldSet dmpReferencesFields = fields.extractPrefixed(this.asIndexerPrefix(Reference._dmpReferences));
        this.censorFactory.censor(DmpReferenceCensor.class).censor(dmpReferencesFields, userId);
        FieldSet createdByFields = fields.extractPrefixed(this.asIndexerPrefix(Reference._createdBy));
        this.censorFactory.censor(UserCensor.class).censor(createdByFields, userId);
        FieldSet typeFields = fields.extractPrefixed(this.asIndexerPrefix(Reference._type));
        this.censorFactory.censor(ReferenceTypeCensor.class).censor(typeFields, userId);
    }

}
