package eu.eudat.model.persist;

import eu.eudat.commons.validation.BaseValidator;
import eu.eudat.convention.ConventionService;
import eu.eudat.errorcode.ErrorThesaurusProperties;
import gr.cite.tools.validation.specification.Specification;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;
import java.util.UUID;

public class DescriptionSectionPermissionResolver {

    private UUID dmpId;
    public static final String _dmpId = "dmpId";

    private List<UUID> sectionIds;

    public static final String _sectionIds = "sectionIds";

    private List<String> permissions;

    public static final String _permissions = "permissions";

    public UUID getDmpId() {
        return dmpId;
    }

    public void setDmpId(UUID dmpId) {
        this.dmpId = dmpId;
    }

    public List<UUID> getSectionIds() {
        return sectionIds;
    }

    public void setSectionIds(List<UUID> sectionIds) {
        this.sectionIds = sectionIds;
    }

    public List<String> getPermissions() {
        return permissions;
    }

    public void setPermissions(List<String> permissions) {
        this.permissions = permissions;
    }

    @Component(DescriptionSectionPermissionResolverPersistValidator.ValidatorName)
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public static class DescriptionSectionPermissionResolverPersistValidator extends BaseValidator<DescriptionSectionPermissionResolver> {

        public static final String ValidatorName = "DescriptionSectionPermissionResolverPersistValidator";

        private final MessageSource messageSource;

        protected DescriptionSectionPermissionResolverPersistValidator(ConventionService conventionService, ErrorThesaurusProperties errors, MessageSource messageSource) {
            super(conventionService, errors);
            this.messageSource = messageSource;
        }

        @Override
        protected Class<DescriptionSectionPermissionResolver> modelClass() {
            return DescriptionSectionPermissionResolver.class;
        }

        @Override
        protected List<Specification> specifications(DescriptionSectionPermissionResolver item) {
            return Arrays.asList(
                    this.spec()
                            .must(() -> this.isValidGuid(item.getDmpId()))
                            .failOn(DescriptionSectionPermissionResolver._dmpId).failWith(messageSource.getMessage("Validation_Required", new Object[]{DescriptionSectionPermissionResolver._dmpId}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .must(() -> item.getPermissions() != null && !item.getPermissions().isEmpty())
                            .failOn(DescriptionSectionPermissionResolver._permissions).failWith(messageSource.getMessage("Validation_Required", new Object[]{DescriptionSectionPermissionResolver._permissions}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .must(() -> item.getSectionIds() != null && !item.getSectionIds().isEmpty())
                            .failOn(DescriptionSectionPermissionResolver._sectionIds).failWith(messageSource.getMessage("Validation_Required", new Object[]{DescriptionSectionPermissionResolver._sectionIds}, LocaleContextHolder.getLocale()))

                    );
        }
    }

}
