package eu.eudat.model.persist;

import eu.eudat.commons.validation.BaseValidator;
import gr.cite.tools.validation.ValidatorFactory;
import gr.cite.tools.validation.specification.Specification;
import eu.eudat.convention.ConventionService;
import eu.eudat.errorcode.ErrorThesaurusProperties;
import eu.eudat.model.persist.dmpreference.DmpReferenceDataPersist;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;

public class DmpReferencePersist {

    private ReferencePersist reference;

    public static final String _reference = "reference";


    private DmpReferenceDataPersist data;

    public static final String _data = "data";


    public ReferencePersist getReference() {
        return reference;
    }

    public void setReference(ReferencePersist reference) {
        this.reference = reference;
    }

    public DmpReferenceDataPersist getData() {
        return data;
    }

    public void setData(DmpReferenceDataPersist data) {
        this.data = data;
    }

    @Component(DmpReferencePersistValidator.ValidatorName)
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public static class DmpReferencePersistValidator extends BaseValidator<DmpReferencePersist> {

        public static final String ValidatorName = "DmpReferencePersistValidator";

        private final MessageSource messageSource;

        private final ValidatorFactory validatorFactory;

        protected DmpReferencePersistValidator(ConventionService conventionService, ErrorThesaurusProperties errors, MessageSource messageSource, ValidatorFactory validatorFactory) {
            super(conventionService, errors);
            this.messageSource = messageSource;
            this.validatorFactory = validatorFactory;
        }

        @Override
        protected Class<DmpReferencePersist> modelClass() {
            return DmpReferencePersist.class;
        }

        @Override
        protected List<Specification> specifications(DmpReferencePersist item) {
            return Arrays.asList(
                    this.spec()
                            .must(() -> !this.isNull(item.getReference()))
                            .failOn(DmpReferencePersist._reference).failWith(messageSource.getMessage("Validation_Required", new Object[]{DmpReferencePersist._reference}, LocaleContextHolder.getLocale())),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getReference()))
                            .on(DmpReferencePersist._reference)
                            .over(item.getReference())
                            .using(() -> this.validatorFactory.validator(ReferencePersist.ReferenceWithoutTypePersistValidator.class)),
                    this.spec()
                            .must(() -> !this.isNull(item.getData()))
                            .failOn(DmpReferencePersist._data).failWith(messageSource.getMessage("Validation_Required", new Object[]{DmpReferencePersist._data}, LocaleContextHolder.getLocale())),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()))
                            .on(DmpReferencePersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(DmpReferenceDataPersist.DmpReferenceDataPersistValidator.class))
            );
        }

    }

}
