package eu.eudat.model.persist;

import eu.eudat.commons.validation.BaseValidator;
import gr.cite.tools.validation.ValidatorFactory;
import gr.cite.tools.validation.specification.Specification;
import eu.eudat.convention.ConventionService;
import eu.eudat.errorcode.ErrorThesaurusProperties;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;

public class DmpUserInvitePersist {

    private List<DmpUserPersist> users;

    public static final String _users = "users";

    public List<DmpUserPersist> getUsers() {
        return users;
    }

    public void setUsers(List<DmpUserPersist> users) {
        this.users = users;
    }

    @Component(DmpUserInvitePersistValidator.ValidatorName)
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public static class DmpUserInvitePersistValidator extends BaseValidator<DmpUserInvitePersist> {

        public static final String ValidatorName = "DmpUserInvitePersistValidator";

        private final MessageSource messageSource;

        private final ValidatorFactory validatorFactory;

        protected DmpUserInvitePersistValidator(ConventionService conventionService, ErrorThesaurusProperties errors, MessageSource messageSource, ValidatorFactory validatorFactory) {
            super(conventionService, errors);
            this.messageSource = messageSource;
            this.validatorFactory = validatorFactory;
        }

        @Override
        protected Class<DmpUserInvitePersist> modelClass() {
            return DmpUserInvitePersist.class;
        }

        @Override
        protected List<Specification> specifications(DmpUserInvitePersist item) {
            return Arrays.asList(
                    this.spec()
                            .must(() -> !this.isListNullOrEmpty(item.getUsers()))
                            .failOn(DmpUserInvitePersist._users).failWith(messageSource.getMessage("Validation_Required", new Object[]{DmpUserInvitePersist._users}, LocaleContextHolder.getLocale())),
                    this.navSpec()
                            .iff(() -> !this.isListNullOrEmpty(item.getUsers()))
                            .on(DmpUserInvitePersist._users)
                            .over(item.getUsers())
                            .using((itm) -> this.validatorFactory.validator(DmpUserPersist.DmpUserPersistValidator.class))
            );
        }
    }

}
