package eu.eudat.model.persist.actionconfirmation;

import eu.eudat.commons.enums.DmpUserRole;
import eu.eudat.commons.validation.BaseValidator;
import gr.cite.tools.validation.specification.Specification;
import eu.eudat.convention.ConventionService;
import eu.eudat.errorcode.ErrorThesaurusProperties;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;
import java.util.UUID;

public class DmpInvitationPersist {

    private String email;

    public static final String _email = "email";

    private UUID dmpId;

    public static final String _dmpId = "dmpId";

    private DmpUserRole role;

    public static final String _role = "role";

    public DmpInvitationPersist(String email, UUID dmpId, DmpUserRole role) {
        this.email = email;
        this.dmpId = dmpId;
        this.role = role;
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public UUID getDmpId() {
        return dmpId;
    }

    public void setDmpId(UUID dmpId) {
        this.dmpId = dmpId;
    }

    public DmpUserRole getRole() {
        return role;
    }

    public void setRole(DmpUserRole role) {
        this.role = role;
    }

    @Component(DmpInvitationPersistValidator.ValidatorName)
    @Scope(value = ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public static class DmpInvitationPersistValidator extends BaseValidator<DmpInvitationPersist> {

        public static final String ValidatorName = "DmpInvitationPersistValidator";

        private final MessageSource messageSource;

        protected DmpInvitationPersistValidator(ConventionService conventionService, ErrorThesaurusProperties errors, MessageSource messageSource) {
            super(conventionService, errors);
            this.messageSource = messageSource;
        }

        @Override
        protected Class<DmpInvitationPersist> modelClass() {
            return DmpInvitationPersist.class;
        }

        @Override
        protected List<Specification> specifications(DmpInvitationPersist item) {
            return Arrays.asList(
                    this.spec()
                            .must(() -> !this.isEmpty(item.getEmail()))
                            .failOn(DmpInvitationPersist._email).failWith(messageSource.getMessage("Validation_Required", new Object[]{DmpInvitationPersist._email}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .must(() -> !this.isNull(item.getDmpId()))
                            .failOn(DmpInvitationPersist._dmpId).failWith(messageSource.getMessage("Validation_Required", new Object[]{DmpInvitationPersist._dmpId}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .must(() -> !this.isNull(item.getRole()))
                            .failOn(DmpInvitationPersist._role).failWith(messageSource.getMessage("Validation_Required", new Object[]{DmpInvitationPersist._role}, LocaleContextHolder.getLocale()))
            );
        }
    }

}
