package eu.eudat.model.persist.descriptiontemplatedefinition.fielddata;

import com.fasterxml.jackson.annotation.JsonSubTypes;
import com.fasterxml.jackson.annotation.JsonTypeInfo;
import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.validation.BaseValidator;
import gr.cite.tools.validation.specification.Specification;
import eu.eudat.convention.ConventionService;
import eu.eudat.errorcode.ErrorThesaurusProperties;
import org.springframework.context.MessageSource;
import org.springframework.context.i18n.LocaleContextHolder;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@JsonTypeInfo(
        use = JsonTypeInfo.Id.NAME,
        include = JsonTypeInfo.As.PROPERTY,
        property = "fieldType",
        visible = true,
        defaultImpl = LabelDataPersist.class)
@JsonSubTypes({
        @JsonSubTypes.Type(value = LabelDataPersist.class, name = FieldType.Names.BooleanDecision),
        @JsonSubTypes.Type(value = LabelAndMultiplicityDataPersist.class, name = FieldType.Names.InternalEntitiesDescriptions),
        @JsonSubTypes.Type(value = LabelAndMultiplicityDataPersist.class, name = FieldType.Names.InternalEntitiesDmps),
        @JsonSubTypes.Type(value = LabelDataPersist.class, name = FieldType.Names.CheckBox),
        @JsonSubTypes.Type(value = LabelDataPersist.class, name = FieldType.Names.DatePicker),
        @JsonSubTypes.Type(value = LabelDataPersist.class, name = FieldType.Names.FreeText),
        @JsonSubTypes.Type(value = ReferenceTypeDataPersist.class, name = FieldType.Names.ReferenceTypes),
        @JsonSubTypes.Type(value = RadioBoxDataPersist.class, name = FieldType.Names.RadioBox),
        @JsonSubTypes.Type(value = LabelDataPersist.class, name = FieldType.Names.RichTextarea),
        @JsonSubTypes.Type(value = LabelDataPersist.class, name = FieldType.Names.Tags),
        @JsonSubTypes.Type(value = LabelDataPersist.class, name = FieldType.Names.TextArea),
        @JsonSubTypes.Type(value = UploadDataPersist.class, name = FieldType.Names.Upload),
        @JsonSubTypes.Type(value = LabelDataPersist.class, name = FieldType.Names.Validation),
        @JsonSubTypes.Type(value = LabelDataPersist.class, name = FieldType.Names.DatasetIdentifier),
        @JsonSubTypes.Type(value = SelectDataPersist.class, name = FieldType.Names.Select)
})
public abstract class BaseFieldDataPersist {

    private String label = null;

    public static final String _label = "label";

    private FieldType fieldType;

    public static final String _fieldType = "fieldType";

    public String getLabel() {
        return label;
    }

    public void setLabel(String label) {
        this.label = label;
    }

    public FieldType getFieldType() {
        return fieldType;
    }

    public void setFieldType(FieldType fieldType) {
        this.fieldType = fieldType;
    }

    public abstract static class BaseFieldDataPersistValidator<T extends BaseFieldDataPersist> extends BaseValidator<T> {

        protected final MessageSource messageSource;

        protected BaseFieldDataPersistValidator(ConventionService conventionService, ErrorThesaurusProperties errors, MessageSource messageSource) {
            super(conventionService, errors);
            this.messageSource = messageSource;
        }

        protected List<Specification> getBaseSpecifications(T item) {
            List<Specification> specifications = new ArrayList<>();
            specifications.addAll(Arrays.asList(
                    this.spec()
                            .must(() -> !this.isNull(item.getFieldType()))
                            .failOn(BaseFieldDataPersist._fieldType).failWith(messageSource.getMessage("Validation_Required", new Object[]{BaseFieldDataPersist._fieldType}, LocaleContextHolder.getLocale()))
            ));
            return specifications;
        }

    }

}
