package eu.eudat.model.persist.dmpblueprintdefinition;

import eu.eudat.commons.enums.DmpBlueprintExtraFieldDataType;
import gr.cite.tools.validation.specification.Specification;
import eu.eudat.convention.ConventionService;
import eu.eudat.errorcode.ErrorThesaurusProperties;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;

public class ExtraFieldPersist extends FieldPersist {

    private DmpBlueprintExtraFieldDataType dataType;

    public static final String _dataType = "dataType";

    public DmpBlueprintExtraFieldDataType getDataType() {
        return dataType;
    }

    public void setDataType(DmpBlueprintExtraFieldDataType dataType) {
        this.dataType = dataType;
    }

    @Component(ExtraFieldPersistValidator.ValidatorName)
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public static class ExtraFieldPersistValidator extends BaseFieldPersistValidator<ExtraFieldPersist> {

        public static final String ValidatorName = "DmpBlueprint.ExtraFieldPersistValidator";

        protected ExtraFieldPersistValidator(ConventionService conventionService, ErrorThesaurusProperties errors, MessageSource messageSource) {
            super(conventionService, errors, messageSource);
        }

        @Override
        protected Class<ExtraFieldPersist> modelClass() {
            return ExtraFieldPersist.class;
        }

        @Override
        protected List<Specification> specifications(ExtraFieldPersist item) {
            List<Specification> specifications = getBaseSpecifications(item);
            specifications.addAll(Arrays.asList(
                    this.spec()
                            .must(() -> !this.isEmpty(item.getLabel()))
                            .failOn(FieldPersist._label).failWith(messageSource.getMessage("Validation_Required", new Object[]{FieldPersist._label}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .must(() -> !this.isNull(item.getDataType()))
                            .failOn(ExtraFieldPersist._dataType).failWith(messageSource.getMessage("Validation_Required", new Object[]{ExtraFieldPersist._dataType}, LocaleContextHolder.getLocale()))
            ));
            return specifications;
        }
    }

}
