package eu.eudat.model.persist.dmpproperties;

import eu.eudat.commons.validation.BaseValidator;
import gr.cite.tools.validation.specification.Specification;
import eu.eudat.convention.ConventionService;
import eu.eudat.errorcode.ErrorThesaurusProperties;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;
import java.util.UUID;

public class DmpContactPersist {

    private UUID userId;
    public static final String _userId = "userId";

    private String firstName;
    public static final String _firstName = "firstName";

    private String lastName;
    public static final String _lastName = "lastName";

    private String email;
    public static final String _email = "email";

    public UUID getUserId() {
        return userId;
    }

    public void setUserId(UUID userId) {
        this.userId = userId;
    }

    public String getFirstName() {
        return firstName;
    }

    public void setFirstName(String firstName) {
        this.firstName = firstName;
    }

    public String getLastName() {
        return lastName;
    }

    public void setLastName(String lastName) {
        this.lastName = lastName;
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    @Component(DmpContactPersistValidator.ValidatorName)
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public static class DmpContactPersistValidator extends BaseValidator<DmpContactPersist> {

        public static final String ValidatorName = "DmpContactPersistValidator";
        private final MessageSource messageSource;

        protected DmpContactPersistValidator(ConventionService conventionService, ErrorThesaurusProperties errors, MessageSource messageSource) {
            super(conventionService, errors);
            this.messageSource = messageSource;
        }

        @Override
        protected Class<DmpContactPersist> modelClass() {
            return DmpContactPersist.class;
        }

        @Override
        protected List<Specification> specifications(DmpContactPersist item) {
            return Arrays.asList(
                    this.spec()
                            .iff(() -> this.isEmpty(item.getEmail()) && this.isEmpty(item.getFirstName()) && this.isEmpty(item.getLastName()))
                            .must(() -> !this.isNull(item.getUserId()))
                            .failOn(DmpContactPersist._userId).failWith(messageSource.getMessage("Validation_Required", new Object[]{"user"}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .iff(() -> this.isNull(item.getUserId()))
                            .must(() -> !this.isEmpty(item.getEmail()))
                            .failOn(DmpContactPersist._email).failWith(messageSource.getMessage("Validation_Required", new Object[]{DmpContactPersist._email}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .iff(() -> this.isNull(item.getUserId()))
                            .must(() -> !this.isEmpty(item.getFirstName()))
                            .failOn(DmpContactPersist._firstName).failWith(messageSource.getMessage("Validation_Required", new Object[]{DmpContactPersist._firstName}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .iff(() -> this.isNull(item.getUserId()))
                            .must(() -> !this.isEmpty(item.getLastName()))
                            .failOn(DmpContactPersist._lastName).failWith(messageSource.getMessage("Validation_Required", new Object[]{DmpContactPersist._lastName}, LocaleContextHolder.getLocale()))
                    );
        }
    }

}
