package eu.eudat.model.persist.dmpproperties;

import eu.eudat.commons.enums.DmpBlueprintFieldCategory;
import eu.eudat.commons.enums.DmpBlueprintSystemFieldType;
import eu.eudat.commons.enums.DmpStatus;
import eu.eudat.commons.types.dmpblueprint.DefinitionEntity;
import eu.eudat.commons.types.dmpblueprint.FieldEntity;
import eu.eudat.commons.types.dmpblueprint.SectionEntity;
import eu.eudat.commons.types.dmpblueprint.SystemFieldEntity;
import eu.eudat.commons.validation.BaseValidator;
import gr.cite.tools.validation.ValidatorFactory;
import gr.cite.tools.validation.specification.Specification;
import eu.eudat.convention.ConventionService;
import eu.eudat.errorcode.ErrorThesaurusProperties;
import eu.eudat.model.persist.validation.StatusAware;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;

public class DmpPropertiesPersist {

    private Map<UUID, DmpBlueprintValuePersist> dmpBlueprintValues;

    public static final String _dmpBlueprintValues = "dmpBlueprintValues";

    private List<DmpContactPersist> contacts;

    public static final String _contacts = "contacts";

    public Map<UUID, DmpBlueprintValuePersist> getDmpBlueprintValues() {
        return dmpBlueprintValues;
    }

    public void setDmpBlueprintValues(Map<UUID, DmpBlueprintValuePersist> dmpBlueprintValues) {
        this.dmpBlueprintValues = dmpBlueprintValues;
    }

    public List<DmpContactPersist> getContacts() {
        return contacts;
    }

    public void setContacts(List<DmpContactPersist> contacts) {
        this.contacts = contacts;
    }

    @Component(DmpPropertiesPersistValidator.ValidatorName)
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public static class DmpPropertiesPersistValidator extends BaseValidator<DmpPropertiesPersist> implements StatusAware<DmpStatus> {

        public static final String ValidatorName = "DmpPropertiesPersistValidator";

        private final ValidatorFactory validatorFactory;
        private final MessageSource messageSource;

        private DmpStatus status;
        private DefinitionEntity definition;

        protected DmpPropertiesPersistValidator(ConventionService conventionService, ErrorThesaurusProperties errors, ValidatorFactory validatorFactory, MessageSource messageSource) {
            super(conventionService, errors);
            this.validatorFactory = validatorFactory;
            this.messageSource = messageSource;
        }

        @Override
        protected Class<DmpPropertiesPersist> modelClass() {
            return DmpPropertiesPersist.class;
        }

        @Override
        protected List<Specification> specifications(DmpPropertiesPersist item) {
            String contactField = this.getContactFieldLabel();
            return Arrays.asList(
                    this.mapSpec()
                            .iff(() ->this.status == DmpStatus.Finalized && !this.isNull(item.getDmpBlueprintValues()))
                            .on(DmpPropertiesPersist._dmpBlueprintValues)
                            .over(item.getDmpBlueprintValues())
                            .mapKey((k) -> ((UUID)k).toString())
                            .using((itm) -> this.validatorFactory.validator(DmpBlueprintValuePersist.DmpBlueprintValuePersistValidator.class).withDefinition(definition)),
                    this.spec()
                            .iff(() -> this.status == DmpStatus.Finalized && contactField != null)
                            .must(() -> !this.isListNullOrEmpty(item.getContacts()))
                            .failOn(DmpPropertiesPersist._contacts).failWith(messageSource.getMessage("Validation_Required", new Object[]{contactField}, LocaleContextHolder.getLocale())),
                    this.navSpec()
                            .iff(() -> this.status == DmpStatus.Finalized && !this.isListNullOrEmpty(item.getContacts()))
                            .on(DmpPropertiesPersist._contacts)
                            .over(item.getContacts())
                            .using((itm) -> this.validatorFactory.validator(DmpContactPersist.DmpContactPersistValidator.class))
            );
        }

        @Override
        public DmpPropertiesPersistValidator setStatus(DmpStatus status) {
            this.status = status;
            return this;
        }

        public DmpPropertiesPersistValidator withDefinition(DefinitionEntity definition) {
            this.definition = definition;
            return this;
        }

        private String getContactFieldLabel(){
            if (this.isListNullOrEmpty(definition.getSections())) return null;

            for (SectionEntity section: definition.getSections()) {
                if (!this.isListNullOrEmpty(section.getFields())) {
                    List<FieldEntity> fields = section.getFields().stream().filter(x -> x.getCategory().equals(DmpBlueprintFieldCategory.System)).collect(Collectors.toList());
                    if (!this.isListNullOrEmpty(fields)){

                        List<SystemFieldEntity> systemFields = (List<SystemFieldEntity>)(List<?>) fields;
                        if (!this.isListNullOrEmpty(systemFields)){
                            List<SystemFieldEntity> contactSystemFields = systemFields.stream().filter(x -> x.getType().equals(DmpBlueprintSystemFieldType.Contact) && x.isRequired()).collect(Collectors.toList());
                            if(!this.isListNullOrEmpty(contactSystemFields)) {
                                return contactSystemFields.getFirst().getLabel() != null ? contactSystemFields.getFirst().getLabel() : DmpBlueprintSystemFieldType.Contact.name();
                            }
                        }

                    }
                }
            }
            return null;
        }

    }

}
