package eu.eudat.model.persist.tenantconfig;

import eu.eudat.commons.validation.BaseValidator;
import gr.cite.tools.validation.ValidatorFactory;
import gr.cite.tools.validation.specification.Specification;
import eu.eudat.convention.ConventionService;
import eu.eudat.errorcode.ErrorThesaurusProperties;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.List;

public class TenantDepositConfigPersist {

    private List<TenantSourcePersist> sources;

    public static final String _sources = "sources";

    public List<TenantSourcePersist> getSources() {
        return sources;
    }

    public void setSources(List<TenantSourcePersist> sources) {
        this.sources = sources;
    }

    @Component(TenantDepositConfigPersistValidator.ValidatorName)
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public static class TenantDepositConfigPersistValidator extends BaseValidator<TenantDepositConfigPersist> {

        public static final String ValidatorName = "TenantDepositConfigPersistValidator";

        private final ValidatorFactory validatorFactory;

        protected TenantDepositConfigPersistValidator(ConventionService conventionService, ErrorThesaurusProperties errors, ValidatorFactory validatorFactory) {
            super(conventionService, errors);
            this.validatorFactory = validatorFactory;
        }

        @Override
        protected Class<TenantDepositConfigPersist> modelClass() {
            return TenantDepositConfigPersist.class;
        }

        @Override
        protected List<Specification> specifications(TenantDepositConfigPersist item) {
            return Collections.singletonList(
                    this.navSpec()
                            .iff(() -> !this.isListNullOrEmpty(item.getSources()))
                            .on(TenantDepositConfigPersist._sources)
                            .over(item.getSources())
                            .using((itm) -> this.validatorFactory.validator(TenantSourcePersist.TenantSourcePersistValidator.class))
            );
        }
    }

}
