package eu.eudat.query;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.authorization.Permission;
import eu.eudat.commons.enums.IsActive;
import eu.eudat.commons.scope.user.UserScope;
import eu.eudat.data.DmpDescriptionTemplateEntity;
import eu.eudat.data.DmpEntity;
import eu.eudat.model.DmpDescriptionTemplate;
import eu.eudat.model.PublicDmpDescriptionTemplate;
import eu.eudat.query.utils.QueryUtilsService;
import gr.cite.commons.web.authz.service.AuthorizationService;
import gr.cite.tools.data.query.FieldResolver;
import gr.cite.tools.data.query.QueryBase;
import gr.cite.tools.data.query.QueryContext;
import jakarta.persistence.Tuple;
import jakarta.persistence.criteria.CriteriaBuilder;
import jakarta.persistence.criteria.Predicate;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.time.Instant;
import java.util.*;

@Component
@Scope(value = ConfigurableBeanFactory.SCOPE_PROTOTYPE)
public class DmpDescriptionTemplateQuery extends QueryBase<DmpDescriptionTemplateEntity> {

    private Collection<UUID> ids;

    private Collection<UUID> dmpIds;

    private Collection<UUID> descriptionTemplateGroupIds;

    private Collection<UUID> sectionIds;

    private Collection<IsActive> isActives;

    private Collection<UUID> excludedIds;

    private Instant after;

    private EnumSet<AuthorizationFlags> authorize = EnumSet.of(AuthorizationFlags.None);

    public DmpDescriptionTemplateQuery ids(UUID value) {
        this.ids = List.of(value);
        return this;
    }

    public DmpDescriptionTemplateQuery ids(UUID... value) {
        this.ids = Arrays.asList(value);
        return this;
    }

    public DmpDescriptionTemplateQuery ids(Collection<UUID> values) {
        this.ids = values;
        return this;
    }

    public DmpDescriptionTemplateQuery excludedIds(Collection<UUID> values) {
        this.excludedIds = values;
        return this;
    }

    public DmpDescriptionTemplateQuery excludedIds(UUID value) {
        this.excludedIds = List.of(value);
        return this;
    }

    public DmpDescriptionTemplateQuery excludedIds(UUID... value) {
        this.excludedIds = Arrays.asList(value);
        return this;
    }

    public DmpDescriptionTemplateQuery isActive(IsActive value) {
        this.isActives = List.of(value);
        return this;
    }

    public DmpDescriptionTemplateQuery isActive(IsActive... value) {
        this.isActives = Arrays.asList(value);
        return this;
    }

    public DmpDescriptionTemplateQuery isActive(Collection<IsActive> values) {
        this.isActives = values;
        return this;
    }

    public DmpDescriptionTemplateQuery sectionIds(UUID value) {
        this.sectionIds = List.of(value);
        return this;
    }

    public DmpDescriptionTemplateQuery sectionIds(UUID... value) {
        this.sectionIds = Arrays.asList(value);
        return this;
    }

    public DmpDescriptionTemplateQuery sectionIds(Collection<UUID> values) {
        this.sectionIds = values;
        return this;
    }

    public DmpDescriptionTemplateQuery descriptionTemplateGroupIds(UUID value) {
        this.descriptionTemplateGroupIds = List.of(value);
        return this;
    }

    public DmpDescriptionTemplateQuery descriptionTemplateGroupIds(UUID... value) {
        this.descriptionTemplateGroupIds = Arrays.asList(value);
        return this;
    }

    public DmpDescriptionTemplateQuery descriptionTemplateGroupIds(Collection<UUID> values) {
        this.descriptionTemplateGroupIds = values;
        return this;
    }

    public DmpDescriptionTemplateQuery dmpIds(UUID value) {
        this.dmpIds = List.of(value);
        return this;
    }

    public DmpDescriptionTemplateQuery dmpIds(UUID... value) {
        this.dmpIds = Arrays.asList(value);
        return this;
    }

    public DmpDescriptionTemplateQuery dmpIds(Collection<UUID> values) {
        this.dmpIds = values;
        return this;
    }

    public DmpDescriptionTemplateQuery after(Instant value) {
        this.after = value;
        return this;
    }

    public DmpDescriptionTemplateQuery authorize(EnumSet<AuthorizationFlags> values) {
        this.authorize = values;
        return this;
    }

    private final UserScope userScope;

    private final AuthorizationService authService;

    private final QueryUtilsService queryUtilsService;

    public DmpDescriptionTemplateQuery(
            UserScope userScope, AuthorizationService authService, QueryUtilsService queryUtilsService) {
        this.userScope = userScope;
        this.authService = authService;
        this.queryUtilsService = queryUtilsService;
    }

    @Override
    protected Class<DmpDescriptionTemplateEntity> entityClass() {
        return DmpDescriptionTemplateEntity.class;
    }

    @Override
    protected Boolean isFalseQuery() {
        return this.isEmpty(this.ids) || this.isEmpty(this.excludedIds) || this.isEmpty(this.isActives) || this.isEmpty(this.dmpIds) || this.isEmpty(this.descriptionTemplateGroupIds);
    }

    @Override
    protected <X, Y> Predicate applyAuthZ(QueryContext<X, Y> queryContext) {
        if (this.authorize.contains(AuthorizationFlags.None))
            return null;
        if (this.authorize.contains(AuthorizationFlags.Permission) && this.authService.authorize(Permission.BrowseDmpDescriptionTemplate))
            return null;
        UUID userId = null;
        boolean usePublic = this.authorize.contains(AuthorizationFlags.Public);
        if (this.authorize.contains(AuthorizationFlags.DmpAssociated))
            userId = this.userScope.getUserIdSafe();

        List<Predicate> predicates = new ArrayList<>();
        if (userId != null || usePublic) {
            predicates.add(queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpDescriptionTemplateEntity._dmpId)).value(queryUtilsService.buildDmpAuthZSubQuery(queryContext.Query, queryContext.CriteriaBuilder, userId, usePublic)));
        }
        if (!predicates.isEmpty()) {
            Predicate[] predicatesArray = predicates.toArray(new Predicate[0]);
            return queryContext.CriteriaBuilder.and(predicatesArray);
        } else {
            return queryContext.CriteriaBuilder.or(); //Creates a false query
        }
    }

    @Override
    protected <X, Y> Predicate applyFilters(QueryContext<X, Y> queryContext) {
        List<Predicate> predicates = new ArrayList<>();
        if (this.ids != null) {
            CriteriaBuilder.In<UUID> inClause = queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpDescriptionTemplateEntity._id));
            for (UUID item : this.ids)
                inClause.value(item);
            predicates.add(inClause);
        }
        if (this.excludedIds != null) {
            CriteriaBuilder.In<UUID> notInClause = queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpDescriptionTemplateEntity._id));
            for (UUID item : this.excludedIds)
                notInClause.value(item);
            predicates.add(notInClause.not());
        }
        if (this.dmpIds != null) {
            CriteriaBuilder.In<UUID> inClause = queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpDescriptionTemplateEntity._dmpId));
            for (UUID item : this.dmpIds)
                inClause.value(item);
            predicates.add(inClause);
        }
        if (this.descriptionTemplateGroupIds != null) {
            CriteriaBuilder.In<UUID> inClause = queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpDescriptionTemplateEntity._descriptionTemplateGroupId));
            for (UUID item : this.descriptionTemplateGroupIds)
                inClause.value(item);
            predicates.add(inClause);
        }
        if (this.sectionIds != null) {
            CriteriaBuilder.In<UUID> inClause = queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpDescriptionTemplateEntity._sectionId));
            for (UUID item : this.sectionIds)
                inClause.value(item);
            predicates.add(inClause);
        }
        if (this.isActives != null) {
            CriteriaBuilder.In<IsActive> inClause = queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpDescriptionTemplateEntity._isActive));
            for (IsActive item : this.isActives)
                inClause.value(item);
            predicates.add(inClause);
        }
        if (this.after != null) {
            Predicate afterClause = queryContext.CriteriaBuilder.greaterThanOrEqualTo(queryContext.Root.get(DmpEntity._createdAt), this.after);
            predicates.add(afterClause);
        }

        if (!predicates.isEmpty()) {
            Predicate[] predicatesArray = predicates.toArray(new Predicate[0]);
            return queryContext.CriteriaBuilder.and(predicatesArray);
        } else {
            return null;
        }
    }

    @Override
    protected DmpDescriptionTemplateEntity convert(Tuple tuple, Set<String> columns) {
        DmpDescriptionTemplateEntity item = new DmpDescriptionTemplateEntity();
        item.setId(QueryBase.convertSafe(tuple, columns, DmpDescriptionTemplateEntity._id, UUID.class));
        item.setTenantId(QueryBase.convertSafe(tuple, columns, DmpDescriptionTemplateEntity._tenantId, UUID.class));
        item.setDmpId(QueryBase.convertSafe(tuple, columns, DmpDescriptionTemplateEntity._dmpId, UUID.class));
        item.setDescriptionTemplateGroupId(QueryBase.convertSafe(tuple, columns, DmpDescriptionTemplateEntity._descriptionTemplateGroupId, UUID.class));
        item.setSectionId(QueryBase.convertSafe(tuple, columns, DmpDescriptionTemplateEntity._sectionId, UUID.class));
        item.setCreatedAt(QueryBase.convertSafe(tuple, columns, DmpDescriptionTemplateEntity._createdAt, Instant.class));
        item.setUpdatedAt(QueryBase.convertSafe(tuple, columns, DmpDescriptionTemplateEntity._updatedAt, Instant.class));
        item.setIsActive(QueryBase.convertSafe(tuple, columns, DmpDescriptionTemplateEntity._isActive, IsActive.class));
        return item;
    }

    @Override
    protected String fieldNameOf(FieldResolver item) {
        if (item.match(DmpDescriptionTemplate._id) || item.match(PublicDmpDescriptionTemplate._id))
            return DmpDescriptionTemplateEntity._id;
        else if (item.prefix(DmpDescriptionTemplate._dmp) || item.prefix(PublicDmpDescriptionTemplate._dmp))
            return DmpDescriptionTemplateEntity._dmpId;
        else if (item.match(DmpDescriptionTemplate._dmp) || item.prefix(PublicDmpDescriptionTemplate._dmp))
            return DmpDescriptionTemplateEntity._dmpId;
        else if (item.prefix(DmpDescriptionTemplate._currentDescriptionTemplate))
            return DmpDescriptionTemplateEntity._descriptionTemplateGroupId;
        else if (item.match(DmpDescriptionTemplate._currentDescriptionTemplate))
            return DmpDescriptionTemplateEntity._descriptionTemplateGroupId;
        else if (item.prefix(DmpDescriptionTemplate._descriptionTemplates))
            return DmpDescriptionTemplateEntity._descriptionTemplateGroupId;
        else if (item.match(DmpDescriptionTemplate._descriptionTemplates))
            return DmpDescriptionTemplateEntity._descriptionTemplateGroupId;
        else if (item.match(DmpDescriptionTemplate._sectionId))
            return DmpDescriptionTemplateEntity._sectionId;
        else if (item.match(DmpDescriptionTemplate._descriptionTemplateGroupId))
            return DmpDescriptionTemplateEntity._descriptionTemplateGroupId;
        else if (item.match(DmpDescriptionTemplate._hash))
            return DmpDescriptionTemplateEntity._updatedAt;
        else if (item.match(DmpDescriptionTemplate._createdAt))
            return DmpDescriptionTemplateEntity._createdAt;
        else if (item.match(DmpDescriptionTemplate._updatedAt))
            return DmpDescriptionTemplateEntity._updatedAt;
        else if (item.match(DmpDescriptionTemplate._isActive))
            return DmpDescriptionTemplateEntity._isActive;
        else if (item.match(DmpDescriptionTemplate._belongsToCurrentTenant))
            return DmpDescriptionTemplateEntity._tenantId;
        else
            return null;
    }

}
