package eu.eudat.query;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.authorization.Permission;
import eu.eudat.commons.enums.*;
import eu.eudat.commons.scope.user.UserScope;
import eu.eudat.data.*;
import eu.eudat.model.Dmp;
import eu.eudat.model.PublicDmp;
import eu.eudat.query.utils.QueryUtilsService;
import gr.cite.commons.web.authz.service.AuthorizationService;
import gr.cite.tools.data.query.FieldResolver;
import gr.cite.tools.data.query.QueryBase;
import gr.cite.tools.data.query.QueryContext;
import jakarta.persistence.Tuple;
import jakarta.persistence.criteria.CriteriaBuilder;
import jakarta.persistence.criteria.Predicate;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.time.Instant;
import java.util.*;

@Component
@Scope(value = ConfigurableBeanFactory.SCOPE_PROTOTYPE)
public class DmpQuery extends QueryBase<DmpEntity> {

    private String like;

    private Collection<UUID> ids;

    private Collection<UUID> creatorIds;

    private Collection<UUID> excludedIds;

    private Collection<IsActive> isActives;

    private Collection<DmpStatus> statuses;

    private Collection<DmpVersionStatus> versionStatuses;

    private Collection<DmpAccessType> accessTypes;

    private Collection<Integer> versions;

    private Collection<UUID> groupIds;

    private Instant after;

    private DmpUserQuery dmpUserQuery;

    private DmpDescriptionTemplateQuery dmpDescriptionTemplateQuery;

    private DmpReferenceQuery dmpReferenceQuery;

    private EntityDoiQuery entityDoiQuery;

    private EnumSet<AuthorizationFlags> authorize = EnumSet.of(AuthorizationFlags.None);

    private final UserScope userScope;

    private final AuthorizationService authService;

    private final QueryUtilsService queryUtilsService;

    public DmpQuery(UserScope userScope, AuthorizationService authService, QueryUtilsService queryUtilsService) {
        this.userScope = userScope;
        this.authService = authService;
        this.queryUtilsService = queryUtilsService;
    }

    public DmpQuery like(String value) {
        this.like = value;
        return this;
    }

    public DmpQuery ids(UUID value) {
        this.ids = List.of(value);
        return this;
    }

    public DmpQuery ids(UUID... value) {
        this.ids = Arrays.asList(value);
        return this;
    }

    public DmpQuery ids(Collection<UUID> values) {
        this.ids = values;
        return this;
    }

    public DmpQuery creatorIds(UUID value) {
        this.creatorIds = List.of(value);
        return this;
    }

    public DmpQuery creatorIds(UUID... value) {
        this.creatorIds = Arrays.asList(value);
        return this;
    }

    public DmpQuery creatorIds(Collection<UUID> values) {
        this.creatorIds = values;
        return this;
    }

    public DmpQuery excludedIds(Collection<UUID> values) {
        this.excludedIds = values;
        return this;
    }

    public DmpQuery excludedIds(UUID value) {
        this.excludedIds = List.of(value);
        return this;
    }

    public DmpQuery excludedIds(UUID... value) {
        this.excludedIds = Arrays.asList(value);
        return this;
    }

    public DmpQuery isActive(IsActive value) {
        this.isActives = List.of(value);
        return this;
    }

    public DmpQuery isActive(IsActive... value) {
        this.isActives = Arrays.asList(value);
        return this;
    }

    public DmpQuery isActive(Collection<IsActive> values) {
        this.isActives = values;
        return this;
    }

    public DmpQuery versionStatuses(DmpVersionStatus value) {
        this.versionStatuses = List.of(value);
        return this;
    }

    public DmpQuery versionStatuses(DmpVersionStatus... value) {
        this.versionStatuses = Arrays.asList(value);
        return this;
    }

    public DmpQuery versionStatuses(Collection<DmpVersionStatus> values) {
        this.versionStatuses = values;
        return this;
    }

    public DmpQuery accessTypes(DmpAccessType value) {
        this.accessTypes = List.of(value);
        return this;
    }

    public DmpQuery accessTypes(DmpAccessType... value) {
        this.accessTypes = Arrays.asList(value);
        return this;
    }

    public DmpQuery accessTypes(Collection<DmpAccessType> values) {
        this.accessTypes = values;
        return this;
    }

    public DmpQuery statuses(DmpStatus value) {
        this.statuses = List.of(value);
        return this;
    }

    public DmpQuery statuses(DmpStatus... value) {
        this.statuses = Arrays.asList(value);
        return this;
    }

    public DmpQuery statuses(Collection<DmpStatus> values) {
        this.statuses = values;
        return this;
    }

    public DmpQuery versions(Integer value) {
        this.versions = List.of(value);
        return this;
    }

    public DmpQuery versions(Integer... value) {
        this.versions = Arrays.asList(value);
        return this;
    }

    public DmpQuery versions(Collection<Integer> values) {
        this.versions = values;
        return this;
    }

    public DmpQuery groupIds(UUID value) {
        this.groupIds = List.of(value);
        return this;
    }

    public DmpQuery groupIds(UUID... value) {
        this.groupIds = Arrays.asList(value);
        return this;
    }

    public DmpQuery groupIds(Collection<UUID> values) {
        this.groupIds = values;
        return this;
    }

    public DmpQuery after(Instant value) {
        this.after = value;
        return this;
    }

    public DmpQuery dmpDescriptionTemplateSubQuery(DmpDescriptionTemplateQuery subQuery) {
        this.dmpDescriptionTemplateQuery = subQuery;
        return this;
    }

    public DmpQuery dmpUserSubQuery(DmpUserQuery subQuery) {
        this.dmpUserQuery = subQuery;
        return this;
    }

    public DmpQuery dmpReferenceSubQuery(DmpReferenceQuery subQuery) {
        this.dmpReferenceQuery = subQuery;
        return this;
    }

    public DmpQuery entityDoiSubQuery(EntityDoiQuery subQuery) {
        this.entityDoiQuery = subQuery;
        this.entityDoiQuery.types(EntityType.DMP);
        return this;
    }

    public DmpQuery authorize(EnumSet<AuthorizationFlags> values) {
        this.authorize = values;
        return this;
    }

    @Override
    protected Boolean isFalseQuery() {
        return this.isEmpty(this.ids) || this.isEmpty(this.creatorIds) || this.isEmpty(this.isActives) || this.isEmpty(this.versionStatuses) || this.isEmpty(this.excludedIds) || this.isEmpty(this.accessTypes) || this.isEmpty(this.statuses) || this.isFalseQuery(this.dmpDescriptionTemplateQuery) || this.isFalseQuery(this.dmpUserQuery);
    }

    @Override
    protected Class<DmpEntity> entityClass() {
        return DmpEntity.class;
    }

    @Override
    protected <X, Y> Predicate applyAuthZ(QueryContext<X, Y> queryContext) {
        if (this.authorize.contains(AuthorizationFlags.None))
            return null;
        if (this.authorize.contains(AuthorizationFlags.Permission) && this.authService.authorize(Permission.BrowseDmp))
            return null;
        UUID userId = null;
        boolean usePublic = this.authorize.contains(AuthorizationFlags.Public);
        if (this.authorize.contains(AuthorizationFlags.DmpAssociated))
            userId = this.userScope.getUserIdSafe();

        List<Predicate> predicates = new ArrayList<>();
        if (userId != null || usePublic) {
            predicates.add(queryContext.CriteriaBuilder.or(
                    usePublic ? queryContext.CriteriaBuilder.and(
                            queryContext.CriteriaBuilder.equal(queryContext.Root.get(DmpEntity._status), DmpStatus.Finalized),
                            queryContext.CriteriaBuilder.equal(queryContext.Root.get(DmpEntity._accessType), DmpAccessType.Public)
                    )
                            : queryContext.CriteriaBuilder.or(),  //Creates a false query
                    userId != null ? queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpEntity._id)).value(this.queryUtilsService.buildDmpUserAuthZSubQuery(queryContext.Query, queryContext.CriteriaBuilder, userId)) : queryContext.CriteriaBuilder.or()  //Creates a false query
            ));
        }
        if (!predicates.isEmpty()) {
            Predicate[] predicatesArray = predicates.toArray(new Predicate[0]);
            return queryContext.CriteriaBuilder.and(predicatesArray);
        } else {
            return queryContext.CriteriaBuilder.or(); //Creates a false query
        }
    }

    @Override
    protected <X, Y> Predicate applyFilters(QueryContext<X, Y> queryContext) {
        List<Predicate> predicates = new ArrayList<>();
        if (this.like != null && !this.like.isEmpty()) {
            predicates.add(queryUtilsService.ilike(queryContext.CriteriaBuilder, queryContext.Root.get(DmpEntity._label), this.like));
        }
        if (this.ids != null) {
            CriteriaBuilder.In<UUID> inClause = queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpEntity._id));
            for (UUID item : this.ids)
                inClause.value(item);
            predicates.add(inClause);
        }
        if (this.creatorIds != null) {
            CriteriaBuilder.In<UUID> inClause = queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpEntity._creatorId));
            for (UUID item : this.creatorIds)
                inClause.value(item);
            predicates.add(inClause);
        }
        if (this.excludedIds != null) {
            CriteriaBuilder.In<UUID> notInClause = queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpEntity._id));
            for (UUID item : this.excludedIds)
                notInClause.value(item);
            predicates.add(notInClause.not());
        }
        if (this.isActives != null) {
            CriteriaBuilder.In<IsActive> inClause = queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpEntity._isActive));
            for (IsActive item : this.isActives)
                inClause.value(item);
            predicates.add(inClause);
        }
        if (this.statuses != null) {
            CriteriaBuilder.In<DmpStatus> inClause = queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpEntity._status));
            for (DmpStatus item : this.statuses)
                inClause.value(item);
            predicates.add(inClause);
        }
        if (this.versionStatuses != null) {
            CriteriaBuilder.In<DmpVersionStatus> inClause = queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpEntity._versionStatus));
            for (DmpVersionStatus item : this.versionStatuses)
                inClause.value(item);
            predicates.add(inClause);
        }
        if (this.accessTypes != null) {
            CriteriaBuilder.In<DmpAccessType> inClause = queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpEntity._accessType));
            for (DmpAccessType item : this.accessTypes)
                inClause.value(item);
            predicates.add(inClause);
        }
        if (this.versions != null) {
            CriteriaBuilder.In<Integer> inClause = queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpEntity._version));
            for (Integer item : this.versions)
                inClause.value(item);
            predicates.add(inClause);
        }
        if (this.groupIds != null) {
            CriteriaBuilder.In<UUID> inClause = queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpEntity._groupId));
            for (UUID item : this.groupIds)
                inClause.value(item);
            predicates.add(inClause);
        }
        if (this.after != null) {
            Predicate afterClause = queryContext.CriteriaBuilder.greaterThanOrEqualTo(queryContext.Root.get(DmpEntity._createdAt), this.after);
            predicates.add(afterClause);
        }

        if (this.dmpUserQuery != null) {
            QueryContext<DmpUserEntity, UUID> subQuery = this.applySubQuery(this.dmpUserQuery, queryContext, UUID.class, root -> root.get(DmpUserEntity._dmpId));
            predicates.add(queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpEntity._id)).value(subQuery.Query));
        }

        if (this.dmpDescriptionTemplateQuery != null) {
            QueryContext<DmpDescriptionTemplateEntity, UUID> subQuery = this.applySubQuery(this.dmpDescriptionTemplateQuery, queryContext, UUID.class, dmpDescriptionTemplateEntityRoot -> dmpDescriptionTemplateEntityRoot.get(DmpDescriptionTemplateEntity._dmpId));
            predicates.add(queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpEntity._id)).value(subQuery.Query));
        }

        if (this.dmpReferenceQuery != null) {
            QueryContext<DmpReferenceEntity, UUID> subQuery = this.applySubQuery(this.dmpReferenceQuery, queryContext, UUID.class, dmpReferenceEntityRoot -> dmpReferenceEntityRoot.get(DmpReferenceEntity._dmpId));
            predicates.add(queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpEntity._id)).value(subQuery.Query));
        }

        if (this.entityDoiQuery != null) {
            QueryContext<EntityDoiEntity, UUID> subQuery = this.applySubQuery(this.entityDoiQuery, queryContext, UUID.class, entityDoiEntityRoot -> entityDoiEntityRoot.get(EntityDoiEntity._entityId));
            predicates.add(queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpEntity._id)).value(subQuery.Query));
        }

        if (!predicates.isEmpty()) {
            Predicate[] predicatesArray = predicates.toArray(new Predicate[0]);
            return queryContext.CriteriaBuilder.and(predicatesArray);
        } else {
            return null;
        }
    }

    @Override
    protected String fieldNameOf(FieldResolver item) {
        if (item.match(Dmp._id) || item.match(PublicDmp._id))
            return DmpEntity._id;
        else if (item.match(Dmp._label) || item.match(PublicDmp._label))
            return DmpEntity._label;
        else if (item.match(Dmp._version) || item.match(PublicDmp._version))
            return DmpEntity._version;
        else if (item.match(Dmp._status))
            return DmpEntity._status;
        else if (item.match(Dmp._properties))
            return DmpEntity._properties;
        else if (item.prefix(Dmp._properties))
            return DmpEntity._properties;
        else if (item.match(Dmp._groupId))
            return DmpEntity._groupId;
        else if (item.match(Dmp._description) || item.match(PublicDmp._description))
            return DmpEntity._description;
        else if (item.match(Dmp._createdAt))
            return DmpEntity._createdAt;
        else if (item.match(Dmp._updatedAt))
            return DmpEntity._updatedAt;
        else if (item.match(Dmp._hash))
            return DmpEntity._updatedAt;
        else if (item.match(Dmp._isActive))
            return DmpEntity._isActive;
        else if (item.match(Dmp._finalizedAt) || item.match(PublicDmp._finalizedAt))
            return DmpEntity._finalizedAt;
        else if (item.match(Dmp._accessType))
            return DmpEntity._accessType;
        else if (item.match(Dmp._creator))
            return DmpEntity._creatorId;
        else if (item.prefix(Dmp._creator))
            return DmpEntity._blueprintId;
        else if (item.match(Dmp._blueprint))
            return DmpEntity._blueprintId;
        else if (item.prefix(Dmp._blueprint))
            return DmpEntity._blueprintId;
        else if (item.match(Dmp._language))
            return DmpEntity._language;
        else if (item.match(Dmp._publicAfter))
            return DmpEntity._publicAfter;
        else if (item.match(Dmp._versionStatus))
            return DmpEntity._versionStatus;
        else if (item.match(Dmp._belongsToCurrentTenant))
            return DmpEntity._tenantId;
        else
            return null;
    }

    @Override
    protected DmpEntity convert(Tuple tuple, Set<String> columns) {
        DmpEntity item = new DmpEntity();
        item.setId(QueryBase.convertSafe(tuple, columns, DmpEntity._id, UUID.class));
        item.setTenantId(QueryBase.convertSafe(tuple, columns, DmpEntity._tenantId, UUID.class));
        item.setLabel(QueryBase.convertSafe(tuple, columns, DmpEntity._label, String.class));
        item.setVersion(QueryBase.convertSafe(tuple, columns, DmpEntity._version, Short.class));
        item.setStatus(QueryBase.convertSafe(tuple, columns, DmpEntity._status, DmpStatus.class));
        item.setVersionStatus(QueryBase.convertSafe(tuple, columns, DmpEntity._versionStatus, DmpVersionStatus.class));
        item.setProperties(QueryBase.convertSafe(tuple, columns, DmpEntity._properties, String.class));
        item.setGroupId(QueryBase.convertSafe(tuple, columns, DmpEntity._groupId, UUID.class));
        item.setDescription(QueryBase.convertSafe(tuple, columns, DmpEntity._description, String.class));
        item.setCreatedAt(QueryBase.convertSafe(tuple, columns, DmpEntity._createdAt, Instant.class));
        item.setUpdatedAt(QueryBase.convertSafe(tuple, columns, DmpEntity._updatedAt, Instant.class));
        item.setIsActive(QueryBase.convertSafe(tuple, columns, DmpEntity._isActive, IsActive.class));
        item.setFinalizedAt(QueryBase.convertSafe(tuple, columns, DmpEntity._finalizedAt, Instant.class));
        item.setAccessType(QueryBase.convertSafe(tuple, columns, DmpEntity._accessType, DmpAccessType.class));
        item.setCreatorId(QueryBase.convertSafe(tuple, columns, DmpEntity._creatorId, UUID.class));
        item.setBlueprintId(QueryBase.convertSafe(tuple, columns, DmpEntity._blueprintId, UUID.class));
        item.setLanguage(QueryBase.convertSafe(tuple, columns, DmpEntity._language, String.class));
        item.setPublicAfter(QueryBase.convertSafe(tuple, columns, DmpEntity._publicAfter, Instant.class));
        return item;
    }

}
