package eu.eudat.service.deposit;

import eu.eudat.commonmodels.models.dmp.DmpModel;
import eu.eudat.depositinterface.repository.DepositClient;
import eu.eudat.depositinterface.repository.DepositConfiguration;
import gr.cite.tools.exception.MyApplicationException;
import gr.cite.tools.logging.LoggerService;
import gr.cite.tools.logging.MapLogEntry;
import org.slf4j.LoggerFactory;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatusCode;
import org.springframework.http.MediaType;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Mono;

public class DepositClientImpl implements DepositClient {
    private static final LoggerService logger = new LoggerService(LoggerFactory.getLogger(DepositClientImpl.class));

    private final WebClient depositClient;

    public DepositClientImpl(WebClient depositClient) {
        this.depositClient = depositClient;
    }


    @Override
    public String deposit(DmpModel dmpDepositModel, String repositoryAccessToken) throws Exception {
        return depositClient.post().uri("", uriBuilder -> uriBuilder.queryParam("authToken", repositoryAccessToken).build()).bodyValue(dmpDepositModel).exchangeToMono(mono ->  mono.statusCode().isError() ? mono.createException().flatMap(Mono::error) : mono.bodyToMono(String.class)).block();
    }

    @Override
    public String authenticate(String code) {
        return depositClient.get().uri("/authenticate/", uriBuilder -> uriBuilder.queryParam("authToken", code).build()).exchangeToMono(mono ->  mono.statusCode().isError() ? mono.createException().flatMap(Mono::error) : mono.bodyToMono(String.class)).block();
    }

    @Override
    public DepositConfiguration getConfiguration() {
        return depositClient.get().uri("/configuration").exchangeToMono(mono ->  mono.statusCode().isError() ? mono.createException().flatMap(Mono::error) : mono.bodyToMono(new ParameterizedTypeReference<DepositConfiguration>() {})).block();
    }

    @Override
    public String getLogo() {
        return depositClient.get().uri("/logo").exchangeToMono(mono ->  mono.statusCode().isError() ? mono.createException().flatMap(Mono::error) : mono.bodyToMono(String.class)).block();
    }
}
