package eu.eudat.service.transformer;

import eu.eudat.commonmodels.models.FileEnvelopeModel;
import eu.eudat.commonmodels.models.description.DescriptionModel;
import eu.eudat.commonmodels.models.dmp.DmpModel;
import eu.eudat.file.transformer.interfaces.FileTransformerClient;
import eu.eudat.file.transformer.interfaces.FileTransformerConfiguration;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Mono;


public class FileTransformerRepository implements FileTransformerClient {

    private final WebClient transformerClient;

    public FileTransformerRepository(WebClient transformerClient) {
        this.transformerClient = transformerClient;
    }

    @Override
    public FileEnvelopeModel exportDmp(DmpModel dmpModel, String format) {
        return transformerClient.post().uri("/export/dmp", uriBuilder -> uriBuilder.queryParam("format", format).build()).bodyValue(dmpModel)
                .exchangeToMono(mono ->  mono.statusCode().isError() ? mono.createException().flatMap(Mono::error) : mono.bodyToMono(FileEnvelopeModel.class)).block();
    }

    @Override
    public DmpModel importDmp(FileEnvelopeModel fileEnvelope) {
        return transformerClient.post().uri("/import/dmp").bodyValue(fileEnvelope)
                .exchangeToMono(mono ->  mono.statusCode().isError() ? mono.createException().flatMap(Mono::error) : mono.bodyToMono(DmpModel.class)).block();
    }

    @Override
    public FileEnvelopeModel exportDescription(DescriptionModel descriptionModel, String format) {
        return transformerClient.post().uri("/export/description", uriBuilder -> uriBuilder.queryParam("format", format).build()).bodyValue(descriptionModel)
                .exchangeToMono(mono ->  mono.statusCode().isError() ? mono.createException().flatMap(Mono::error) : mono.bodyToMono(FileEnvelopeModel.class)).block();
    }

    @Override
    public DescriptionModel importDescription(FileEnvelopeModel fileEnvelope) {
        return transformerClient.post().uri("/import/description").bodyValue(fileEnvelope)
                .exchangeToMono(mono ->  mono.statusCode().isError() ? mono.createException().flatMap(Mono::error) : mono.bodyToMono(DescriptionModel.class)).block();
    }

    @Override
    public FileTransformerConfiguration getConfiguration() {
        return transformerClient.get().uri("/formats")
                .exchangeToMono(mono ->  mono.statusCode().isError() ? mono.createException().flatMap(Mono::error) : mono.bodyToMono(new ParameterizedTypeReference<FileTransformerConfiguration>() {})).block();
    }


}
