import { Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges } from '@angular/core';
import { DmpBlueprintStatus } from '@app/core/common/enum/dmp-blueprint-status';
import { IsActive } from '@app/core/common/enum/is-active.enum';
import { DmpBlueprintFilter } from '@app/core/query/dmp-blueprint.lookup';
import { EnumUtils } from '@app/core/services/utilities/enum-utils.service';
import { BaseComponent } from '@common/base/base.component';
import { Guid } from '@common/types/guid';
import { nameof } from 'ts-simple-nameof';

@Component({
	selector: 'app-dmp-blueprint-listing-filters',
	templateUrl: './dmp-blueprint-listing-filters.component.html',
	styleUrls: ['./dmp-blueprint-listing-filters.component.scss']
})
export class DmpBlueprintListingFiltersComponent extends BaseComponent implements OnInit, OnChanges {

	@Input() readonly filter: DmpBlueprintFilter;
	@Output() filterChange = new EventEmitter<DmpBlueprintFilter>();

	dmpBlueprintStatusEnumValues = this.enumUtils.getEnumValues<DmpBlueprintStatus>(DmpBlueprintStatus);

	// * State
	internalFilters: DmpBlueprintListingFilters = this._getEmptyFilters();

	protected appliedFilterCount: number = 0;
	constructor(
		public enumUtils: EnumUtils,
	) { super(); }

	ngOnInit() {
	}

	ngOnChanges(changes: SimpleChanges): void {
		const filterChange = changes[nameof<DmpBlueprintListingFiltersComponent>(x => x.filter)]?.currentValue as DmpBlueprintFilter;
		if (filterChange) {
			this.updateFilters()
		}
	}


	onSearchTermChange(searchTerm: string): void {
		this.applyFilters()
	}


	protected updateFilters(): void {
		this.internalFilters = this._parseToInternalFilters(this.filter);
		this.appliedFilterCount = this._computeAppliedFilters(this.internalFilters);
	}

	protected applyFilters(): void {
		const { isActive, like, statuses } = this.internalFilters ?? {}
		this.filterChange.emit({
			...this.filter,
			like,
			isActive: isActive ? [IsActive.Active] : [IsActive.Inactive],
			statuses
		})
	}


	private _parseToInternalFilters(inputFilter: DmpBlueprintFilter): DmpBlueprintListingFilters {
		if (!inputFilter) {
			return this._getEmptyFilters();
		}

		let { excludedIds, ids, isActive, like, statuses } = inputFilter;

		return {
			isActive: (isActive ?? [])?.includes(IsActive.Active) || !isActive?.length,
			like: like,
			statuses: statuses
		}

	}

	private _getEmptyFilters(): DmpBlueprintListingFilters {
		return {
			isActive: true,
			like: null,
			statuses: null,
		}
	}

	private _computeAppliedFilters(filters: DmpBlueprintListingFilters): number {
		let count = 0;
		if (filters?.isActive) {
			count++
		}
		return count;
	}

	clearFilters() {
		this.internalFilters = this._getEmptyFilters();
	}
}

interface DmpBlueprintListingFilters {
	isActive: boolean;
	like: string;
	statuses: DmpBlueprintStatus[];
}
