import { HttpClient } from '@angular/common/http';
import { Component, Inject, OnInit, ViewChild } from '@angular/core';
import { UntypedFormGroup } from '@angular/forms';
import { MAT_DIALOG_DATA, MatDialogRef } from '@angular/material/dialog';
import { DatasetCriteria } from '@app/core/query/dataset/dataset-criteria';
import { MatomoService } from '@app/core/services/matomo/matomo-service';
import { DatasetCriteriaComponent } from '../dataset-criteria.component';

@Component({
	selector: 'dataset-criteria-dialog-component',
	templateUrl: './dataset-criteria-dialog.component.html',
	styleUrls: ['./dataset-criteria-dialog.component.scss']
})

export class DatasetCriteriaDialogComponent implements OnInit {

	@ViewChild(DatasetCriteriaComponent, { static: true }) criteria: DatasetCriteriaComponent;

	constructor(
		public dialogRef: MatDialogRef<DatasetCriteriaDialogComponent>,
		private httpClient: HttpClient,
		private matomoService: MatomoService,
		@Inject(MAT_DIALOG_DATA) public data: { isPublic: boolean, status: Number, criteria: DatasetCriteria, formGroup: UntypedFormGroup, updateDataFn: Function }
	) {
	}

	ngOnInit() {
		this.matomoService.trackPageView('Dataset Criteria');
		this.criteria.setCriteria(this.data.criteria);
	}

	onNoClick(): void {
		this.dialogRef.close();
	}

	onClose(): void {
		this.dialogRef.close();
	}

	onFiltersChanged(event) {
		this.data.updateDataFn(this.criteria);
	}

}
