import { Component, OnInit } from '@angular/core';
import { BaseComponent } from '@common/base/base.component';
// import { BreadcrumbItem } from '@app/ui/misc/breadcrumb/definition/breadcrumb-item';
import { Location } from '@angular/common';
import { UntypedFormBuilder, UntypedFormControl, Validators } from '@angular/forms';
import { MatDialog } from '@angular/material/dialog';
import { ActivatedRoute, Params, Router } from '@angular/router';
import { DescriptionStatus } from '@app/core/common/enum/description-status';
import { DmpAccessType } from '@app/core/common/enum/dmp-access-type';
import { DmpStatus } from '@app/core/common/enum/dmp-status';
import { DmpUserRole } from '@app/core/common/enum/dmp-user-role';
import { DescriptionTemplate } from '@app/core/model/description-template/description-template';
import { Description, DescriptionStatusPersist } from '@app/core/model/description/description';
import { Dmp, DmpDescriptionTemplate, DmpUser, DmpUserRemovePersist } from '@app/core/model/dmp/dmp';
import { DmpReference } from '@app/core/model/dmp/dmp-reference';
import { Reference } from '@app/core/model/reference/reference';
import { AuthService } from '@app/core/services/auth/auth.service';
import { ConfigurationService } from '@app/core/services/configuration/configuration.service';
import { DescriptionService } from '@app/core/services/description/description.service';
import { DmpService } from '@app/core/services/dmp/dmp.service';
import { FileTransformerService } from '@app/core/services/file-transformer/file-transformer.service';
import { MatomoService } from '@app/core/services/matomo/matomo-service';
import { SnackBarNotificationLevel, UiNotificationService } from '@app/core/services/notification/ui-notification-service';
import { ReferenceTypeService } from '@app/core/services/reference-type/reference-type.service';
import { ReferenceService } from '@app/core/services/reference/reference.service';
import { EnumUtils } from '@app/core/services/utilities/enum-utils.service';
import { FileUtils } from '@app/core/services/utilities/file-utils.service';
import { ConfirmationDialogComponent } from '@common/modules/confirmation-dialog/confirmation-dialog.component';
import { Guid } from '@common/types/guid';
import { TranslateService } from '@ngx-translate/core';
import * as FileSaver from 'file-saver';
import { filter, takeUntil } from 'rxjs/operators';
import { nameof } from 'ts-simple-nameof';
import { DescriptionCopyDialogComponent } from '../description-copy-dialog/description-copy-dialog.component';
import { ReferenceType } from '@app/core/model/reference-type/reference-type';
import { AppPermission } from '@app/core/common/enum/permission.enum';
import { DescriptionValidationOutput } from '@app/ui/dmp/dmp-finalize-dialog/dmp-finalize-dialog.component';
import { LockService } from '@app/core/services/lock/lock.service';
import { PopupNotificationDialogComponent } from '@app/library/notification/popup/popup-notification.component';
import { IsActive } from '@app/core/common/enum/is-active.enum';
import { DmpInvitationDialogComponent } from '@app/ui/dmp/invitation/dialog/dmp-invitation-dialog.component';
import { DmpBlueprint, DmpBlueprintDefinition, DmpBlueprintDefinitionSection } from '@app/core/model/dmp-blueprint/dmp-blueprint';


@Component({
	selector: 'app-description-overview',
	templateUrl: './description-overview.component.html',
	styleUrls: ['./description-overview.component.scss']
})
export class DescriptionOverviewComponent extends BaseComponent implements OnInit {

	description: any;
	researchers: DmpReference[] = [];
	isNew = true;
	isFinalized = false;
	isPublicView = true;
	hasPublishButton: boolean = true;
	// breadCrumbs: Observable<BreadcrumbItem[]> = observableOf();
	expand = false;
	isLocked: Boolean;
	descriptionStatusEnum = DescriptionStatus;
	dmpAccessTypeEnum = DmpAccessType;
	dmpStatusEnum = DmpStatus;
	dmpUserRoleEnum = DmpUserRole;

	canEdit = false;
	canDelete = false;
	canFinalize = false;
	canInviteDmpUsers = false;

	constructor(
		private route: ActivatedRoute,
		private router: Router,
		private descriptionService: DescriptionService,
		private authentication: AuthService,
		private dialog: MatDialog,
		private language: TranslateService,
		private uiNotificationService: UiNotificationService,
		private configurationService: ConfigurationService,
		private dmpService: DmpService,
		public referenceService: ReferenceService,
		private location: Location,
		public enumUtils: EnumUtils,
		private matomoService: MatomoService,
		private fileUtils: FileUtils,
		private authService: AuthService,
		public fileTransformerService: FileTransformerService,
		private referenceTypeService: ReferenceTypeService,
		private fb: UntypedFormBuilder,
		private lockService: LockService
	) {
		super();
	}

	ngOnInit() {
		this.matomoService.trackPageView('Description Overview');
		this.canDelete = false;
		this.canEdit = false;
		this.canFinalize = false;
		this.canInviteDmpUsers = false;
		// Gets description data using parameter id
		this.route.params
			.pipe(takeUntil(this._destroyed))
			.subscribe((params: Params) => {
				const itemId = params['id'];
				const publicId = params['publicId'];
				if (itemId != null) {
					this.isNew = false;
					this.isPublicView = false;
					this.descriptionService.getSingle(itemId, this.lookupFields())
						.pipe(takeUntil(this._destroyed))
						.subscribe(data => {
							this.description = data;
							this.description.dmp.dmpUsers = data.dmp.dmpUsers.filter(x => x.isActive === IsActive.Active);
							this.researchers = this.referenceService.getReferencesForTypes(this.description?.dmp?.dmpReferences, [this.referenceTypeService.getResearcherReferenceType()]);
							// this.users = this.description.dmp.users;
							this.checkLockStatus(this.description.id);
							this.canDelete = (this.authService.hasPermission(AppPermission.DeleteDescription) ||
								this.description.authorizationFlags?.some(x => x === AppPermission.DeleteDescription))  && this.description.belongsToCurrentTenant != false;

							this.canEdit = (this.authService.hasPermission(AppPermission.EditDescription) ||
								this.description.authorizationFlags?.some(x => x === AppPermission.EditDescription))  && this.description.belongsToCurrentTenant != false;

							this.canFinalize = (this.authService.hasPermission(AppPermission.FinalizeDescription) ||
								this.description.authorizationFlags?.some(x => x === AppPermission.FinalizeDescription))  && this.description.belongsToCurrentTenant != false;

							this.canInviteDmpUsers = (this.authService.hasPermission(AppPermission.InviteDmpUsers) ||
								this.description.authorizationFlags?.some(x => x === AppPermission.InviteDmpUsers))   && this.description.belongsToCurrentTenant != false;
							// const breadCrumbs = [];
							// breadCrumbs.push({ parentComponentName: null, label: this.language.instant('NAV-BAR.MY-DESCRIPTION-DESCRIPTIONS'), url: "/descriptions" });
							// breadCrumbs.push({ parentComponentName: 'DescriptionListingComponent', label: this.description.label, url: '/descriptions/overview/' + this.description.id });
							// this.breadCrumbs = observableOf(breadCrumbs);

						}, (error: any) => {
							if (error.status === 404) {
								return this.onFetchingDeletedCallbackError('/descriptions/');
							}
							if (error.status === 403) {
								return this.onFetchingForbiddenCallbackError('/descriptions/');
							}
						});
				}
				else if (publicId != null) {
					this.isNew = false;
					this.isFinalized = true;
					this.isPublicView = true;
					this.descriptionService.getPublicSingle(publicId, this.lookupFields())
						.pipe(takeUntil(this._destroyed))
						.subscribe(data => {
							this.description = data;
							this.researchers = this.referenceService.getReferencesForTypes(this.description?.dmp?.dmpReferences, [this.referenceTypeService.getResearcherReferenceType()]);
							// this.users = this.description.dmp.users;
							// const breadCrumbs = [];
							// breadCrumbs.push({ parentComponentName: null, label: this.language.instant('NAV-BAR.PUBLIC DESCRIPTIONS'), url: "/explore" });
							// breadCrumbs.push({ parentComponentName: 'DescriptionListingComponent', label: this.description.label, url: '/descriptions/overview/public/' + this.description.id });
							// this.breadCrumbs = observableOf(breadCrumbs);
						}, (error: any) => {
							if (error.status === 404) {
								return this.onFetchingDeletedCallbackError('/explore-descriptions');
							}
							if (error.status === 403) {
								return this.onFetchingForbiddenCallbackError('/explore-descriptions');
							}
						});
				}
			});
	}

	checkLockStatus(id: Guid) {
		this.lockService.checkLockStatus(id).pipe(takeUntil(this._destroyed))
			.subscribe(lockStatus => {
				this.isLocked = lockStatus.status;
				if (this.isLocked) {
					this.dialog.open(PopupNotificationDialogComponent, {
						data: {
							title: this.language.instant('DESCRIPTION-OVERVIEW.LOCKED-DIALOG.TITLE'),
							message: this.language.instant('DESCRIPTION-OVERVIEW.LOCKED-DIALOG.MESSAGE')
						}, maxWidth: '30em'
					});
				}
			});
	}

	onFetchingDeletedCallbackError(redirectRoot: string) {
		this.uiNotificationService.snackBarNotification(this.language.instant('DESCRIPTION-OVERVIEW.ERROR.DELETED-DESCRIPTION'), SnackBarNotificationLevel.Error);
		this.router.navigate([redirectRoot]);
	}

	onFetchingForbiddenCallbackError(redirectRoot: string) {
		this.uiNotificationService.snackBarNotification(this.language.instant('DESCRIPTION-OVERVIEW.ERROR.FORBIDEN-DESCRIPTION'), SnackBarNotificationLevel.Error);
		this.router.navigate([redirectRoot]);
	}

	goBack(): void {
		this.location.back();
	}

	reloadPage(): void {
		const path = this.location.path();
		this.router.navigateByUrl('/reload', { skipLocationChange: true }).then(() => this.router.navigate([path]));
	}


	isUserAuthor(userId: Guid): boolean {
		if (this.isAuthenticated()) {
			const principalId: Guid = this.authentication.userId();
			return userId === principalId;
		} else return false;
	}


	openShareDialog() {
		const dialogRef = this.dialog.open(DmpInvitationDialogComponent, {
			autoFocus: false,
			restoreFocus: false,
			data: {
				dmpId: this.description.dmp.id,
				dmpName: this.description.dmp.label,
				blueprint: this.description.dmp.blueprint
			}
		});
	}

	public isAuthenticated(): boolean {
		return this.authentication.currentAccountIsAuthenticated();
	}

	isDraftDescription(description: Description) {
		return description.status == DescriptionStatus.Draft;
	}

	editClicked(description: Description) {
		if (description.status === DescriptionStatus.Finalized && description.dmp.accessType === DmpAccessType.Public) {
			this.router.navigate(['/descriptions/publicEdit/', description.id]);
		} else {
			this.router.navigate(['/descriptions/edit/', description.id]);
		}
	}

	deleteClicked() {
		const dialogRef = this.dialog.open(ConfirmationDialogComponent, {
			maxWidth: '300px',
			data: {
				message: this.language.instant('GENERAL.CONFIRMATION-DIALOG.DELETE-ITEM'),
				confirmButton: this.language.instant('GENERAL.CONFIRMATION-DIALOG.ACTIONS.DELETE'),
				cancelButton: this.language.instant('GENERAL.CONFIRMATION-DIALOG.ACTIONS.CANCEL'),
				isDeleteConfirmation: true
			}
		});
		dialogRef.afterClosed().pipe(takeUntil(this._destroyed)).subscribe(result => {
			if (result) {
				this.descriptionService.delete(this.description.id)
					.pipe(takeUntil(this._destroyed))
					.subscribe(
						complete => {
							this.onDeleteCallbackSuccess();
						},
						error => this.onDeleteCallbackError(error)
					);
			}
		});
	}

	dmpClicked(dmp: Dmp) {
		if (this.isPublicView) {
			this.router.navigate(['/explore-plans/overview/public/' + dmp.id]);
		} else {
			this.router.navigate(['/plans/overview/' + dmp.id]);
		}
	}

	private onDeleteCallbackSuccess(): void {
		this.uiNotificationService.snackBarNotification(this.language.instant('GENERAL.SNACK-BAR.SUCCESSFUL-DELETE'), SnackBarNotificationLevel.Success);
		this.router.navigate(['/descriptions']);
	}

	private onDeleteCallbackError(error) {
		this.uiNotificationService.snackBarNotification(error.error.message ? error.error.message : this.language.instant('GENERAL.SNACK-BAR.UNSUCCESSFUL-DELETE'), SnackBarNotificationLevel.Error);
	}

	private onUpdateCallbackSuccess(): void {
		this.uiNotificationService.snackBarNotification(this.language.instant('GENERAL.SNACK-BAR.SUCCESSFUL-UPDATE'), SnackBarNotificationLevel.Success);
		this.reloadPage();
	}

	private onUpdateCallbackError(error) {
		this.uiNotificationService.snackBarNotification(error.error.message ? this.tryTranslate(error.error.message) : this.language.instant('DESCRIPTION-UPLOAD.SNACK-BAR.UNSUCCESSFUL'), SnackBarNotificationLevel.Error);
	}

	private tryTranslate(errorMessage: string): string {
		return errorMessage.replace('Field value of', this.language.instant('Field value of'))
			.replace('must be filled', this.language.instant('must be filled'));
	}

	public getOrcidPath(): string {
		return this.configurationService.orcidPath;
	}

	isOrcid(reference: Reference) {
		return reference.source === 'orcid';
	}

	getOrcidPathForResearcher(reference: string): string {
		const path = this.getOrcidPath();
		return path + reference;
	}

	// downloadPDF(id: string) {
	// 	this.descriptionService.downloadPDF(id)
	// 		.pipe(takeUntil(this._destroyed))
	// 		.subscribe(response => {
	// 			const blob = new Blob([response.body], { type: 'application/pdf' });
	// 			const filename = this.fileUtils.getFilenameFromContentDispositionHeader(response.headers.get('Content-Disposition'));

	// 			FileSaver.saveAs(blob, filename);
	// 			this.matomoService.trackDownload('descriptions', "pdf", id);
	// 		});
	// }

	download(id: string, format: string) {
		this.descriptionService.download(id, format)
			.pipe(takeUntil(this._destroyed))
			.subscribe(response => {
				const blob = new Blob([response.body], { type: 'application/octet-stream' });
				const filename = this.fileUtils.getFilenameFromContentDispositionHeader(response.headers.get('Content-Disposition'));

				FileSaver.saveAs(blob, filename);
				this.matomoService.trackDownload('descriptions', format, id);
			});
	}

	downloadXml(id: string) {
		// TODO: add download
		// this.descriptionService.downloadXML(id)
		// 	.pipe(takeUntil(this._destroyed))
		// 	.subscribe(response => {
		// 		const blob = new Blob([response.body], { type: 'application/xml' });
		// 		const filename = this.fileUtils.getFilenameFromContentDispositionHeader(response.headers.get('Content-Disposition'));

		// 		FileSaver.saveAs(blob, filename);
		// 		this.matomoService.trackDownload('descriptions', "xml", id);
		// 	});
	}

	openCopyToDmpDialog() {
		const formGroup = this.fb.group({
			dmpId: this.fb.control(null, Validators.required),
			sectionId: this.fb.control(null, Validators.required),
		})
		const dialogRef = this.dialog.open(DescriptionCopyDialogComponent, {
			width: '500px',
			restoreFocus: false,
			data: {
				formGroup: formGroup,
				descriptionId: this.description.id,
				descriptionTemplate: this.description.descriptionTemplate,
				dmpDescriptionTemplate: this.description.dmpDescriptionTemplate,
				descriptionProfileExist: false,
				confirmButton: this.language.instant('DESCRIPTION-OVERVIEW.COPY-DIALOG.COPY'),
				cancelButton: this.language.instant('DESCRIPTION-OVERVIEW.COPY-DIALOG.CANCEL')
			}
		});

		dialogRef.afterClosed().pipe(takeUntil(this._destroyed))
			.subscribe(formGroup => {
				if (formGroup) {
					this.router.navigate(['descriptions/edit/copy/' + this.description.id + '/' + formGroup.get('dmpId').value + '/' + formGroup.get('sectionId').value]);
				}
			});
	}

	removeUserFromDmp(dmpUser: DmpUser) {
		const dialogRef = this.dialog.open(ConfirmationDialogComponent, {
			data: {
				message: this.language.instant('GENERAL.CONFIRMATION-DIALOG.DELETE-USER'),
				confirmButton: this.language.instant('GENERAL.CONFIRMATION-DIALOG.ACTIONS.REMOVE'),
				cancelButton: this.language.instant('GENERAL.CONFIRMATION-DIALOG.ACTIONS.CANCEL'),
				isDeleteConfirmation: false
			}
		});
		dialogRef.afterClosed().subscribe(result => {
			if (result) {
				const dmpUserRemovePersist: DmpUserRemovePersist = {
					id: dmpUser.id,
					dmpId: this.description.dmp.id,
					role: dmpUser.role
				};
				this.dmpService.removeUser(dmpUserRemovePersist).pipe(takeUntil(this._destroyed))
					.subscribe(data => {
						this.reloadPage();
					}, (error: any) => {
						//TODO: Error handling
					});

			}
		});
	}

	finalize(description: Description) {

		this.descriptionService.validate([description.id]).pipe(takeUntil(this._destroyed)
			).subscribe(result => {
				if (result[0].result == DescriptionValidationOutput.Invalid){
					this.router.navigate(['descriptions/edit/' + description.id + '/finalize']);
				}else{
					const dialogRef = this.dialog.open(ConfirmationDialogComponent, {
						restoreFocus: false,
						data: {
							message: this.language.instant('DESCRIPTION-OVERVIEW.FINALIZE-DIALOG.TITLE'),
							confirmButton: this.language.instant('DESCRIPTION-OVERVIEW.FINALIZE-DIALOG.CONFIRM'),
							cancelButton: this.language.instant('DESCRIPTION-OVERVIEW.FINALIZE-DIALOG.NEGATIVE'),
							isDeleteConfirmation: false
						}
					});
					dialogRef.afterClosed().pipe(takeUntil(this._destroyed)).subscribe(result => {
						if (result) {
							const descriptionStatusPersist: DescriptionStatusPersist = {
								id: description.id,
								status: DescriptionStatus.Finalized,
								hash: description.hash
							};
							this.descriptionService.persistStatus(descriptionStatusPersist).pipe(takeUntil(this._destroyed))
								.subscribe(data => {
									this.reloadPage();
									this.onUpdateCallbackSuccess()
								}, (error: any) => {
									this.onUpdateCallbackError(error)
								});
						}});
				}
		});
	}

	hasReversableStatus(description: Description): boolean {
		return description.dmp.status == DmpStatus.Draft && description.status == DescriptionStatus.Finalized && this.canFinalize
	}

	reverseFinalization(description: Description) {
		const dialogRef = this.dialog.open(ConfirmationDialogComponent, {
			restoreFocus: false,
			data: {
				message: this.language.instant('DESCRIPTION-OVERVIEW.UNDO-FINALIZATION-DIALOG.TITLE'),
				confirmButton: this.language.instant('DESCRIPTION-OVERVIEW.UNDO-FINALIZATION-DIALOG.CONFIRM'),
				cancelButton: this.language.instant('DESCRIPTION-OVERVIEW.UNDO-FINALIZATION-DIALOG.NEGATIVE'),
				isDeleteConfirmation: false
			}
		});
		dialogRef.afterClosed().pipe(takeUntil(this._destroyed)).subscribe(result => {
			if (result) {
				const dmpUserRemovePersist: DescriptionStatusPersist = {
					id: description.id,
					status: DescriptionStatus.Draft,
					hash: description.hash
				};
				this.descriptionService.persistStatus(dmpUserRemovePersist).pipe(takeUntil(this._destroyed))
					.subscribe(data => {
						this.reloadPage();
						this.onUpdateCallbackSuccess()
					}, (error: any) => {
						this.onUpdateCallbackError(error)
					});
			}
		});
	}

	private lookupFields(): string[] {
		return [
			nameof<Description>(x => x.id),
			nameof<Description>(x => x.label),
			nameof<Description>(x => x.description),
			nameof<Description>(x => x.status),
			nameof<Description>(x => x.updatedAt),
			nameof<Description>(x => x.belongsToCurrentTenant),
			nameof<Description>(x => x.hash),

			[nameof<Description>(x => x.authorizationFlags), AppPermission.EditDescription].join('.'),
			[nameof<Description>(x => x.authorizationFlags), AppPermission.DeleteDescription].join('.'),
			[nameof<Description>(x => x.authorizationFlags), AppPermission.FinalizeDescription].join('.'),
			[nameof<Description>(x => x.authorizationFlags), AppPermission.InviteDmpUsers].join('.'),

			[nameof<Description>(x => x.descriptionTemplate), nameof<DescriptionTemplate>(x => x.id)].join('.'),
			[nameof<Description>(x => x.descriptionTemplate), nameof<DescriptionTemplate>(x => x.label)].join('.'),
			[nameof<Description>(x => x.descriptionTemplate), nameof<DescriptionTemplate>(x => x.groupId)].join('.'),
			[nameof<Description>(x => x.dmpDescriptionTemplate), nameof<DmpDescriptionTemplate>(x => x.id)].join('.'),
			[nameof<Description>(x => x.dmpDescriptionTemplate), nameof<DmpDescriptionTemplate>(x => x.dmp), nameof<Dmp>(x => x.id)].join('.'),
			[nameof<Description>(x => x.dmpDescriptionTemplate), nameof<DmpDescriptionTemplate>(x => x.sectionId)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.id)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.label)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.accessType)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.status)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.authorizationFlags),AppPermission.InviteDmpUsers].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.blueprint), nameof<DmpBlueprint>(x => x.id)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.blueprint), nameof<DmpBlueprint>(x => x.label)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.blueprint), nameof<DmpBlueprint>(x => x.definition), nameof<DmpBlueprintDefinition>(x => x.sections), nameof<DmpBlueprintDefinitionSection>(x => x.id)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.blueprint), nameof<DmpBlueprint>(x => x.definition), nameof<DmpBlueprintDefinition>(x => x.sections), nameof<DmpBlueprintDefinitionSection>(x => x.label)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.dmpUsers), nameof<DmpUser>(x => x.id)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.dmpUsers), nameof<DmpUser>(x => x.user.id)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.dmpUsers), nameof<DmpUser>(x => x.user.name)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.dmpUsers), nameof<DmpUser>(x => x.role)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.dmpUsers), nameof<DmpUser>(x => x.isActive)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.dmpReferences), nameof<DmpReference>(x => x.id)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.dmpReferences), nameof<DmpReference>(x => x.reference), nameof<Reference>(x => x.id)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.dmpReferences), nameof<DmpReference>(x => x.reference), nameof<Reference>(x => x.label)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.dmpReferences), nameof<DmpReference>(x => x.reference), nameof<Reference>(x => x.type), nameof<ReferenceType>(x => x.id)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.dmpReferences), nameof<DmpReference>(x => x.reference), nameof<Reference>(x => x.source)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.dmpReferences), nameof<DmpReference>(x => x.reference), nameof<Reference>(x => x.reference)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.dmpReferences), nameof<DmpReference>(x => x.isActive)].join('.'),
		]
	}

}
