import { DatePipe } from '@angular/common';
import { Component, Inject } from '@angular/core';
import { UntypedFormGroup } from '@angular/forms';
import { MAT_DIALOG_DATA, MatDialog, MatDialogRef } from '@angular/material/dialog';
import { DmpStatus } from '@app/core/common/enum/dmp-status';
import { IsActive } from '@app/core/common/enum/is-active.enum';
import { DescriptionTemplatesInSection, DmpBlueprint, DmpBlueprintDefinition, DmpBlueprintDefinitionSection } from '@app/core/model/dmp-blueprint/dmp-blueprint';
import { Dmp } from '@app/core/model/dmp/dmp';
import { DmpDescriptionTemplateLookup } from '@app/core/query/dmp-description-template.lookup';
import { DmpLookup } from '@app/core/query/dmp.lookup';
import { DmpService } from '@app/core/services/dmp/dmp.service';
import { SingleAutoCompleteConfiguration } from '@app/library/auto-complete/single/single-auto-complete-configuration';
import { BaseComponent } from '@common/base/base.component';
import { FilterService } from '@common/modules/text-filter/filter-service';
import { Guid } from '@common/types/guid';
import { TranslateService } from '@ngx-translate/core';
import { map } from 'rxjs/operators';
import { nameof } from 'ts-simple-nameof';

@Component({
	selector: 'app-start-new-description-dialog',
	templateUrl: './start-new-description-dialog.component.html',
	styleUrls: ['./start-new-description-dialog.component.scss']
})
export class StartNewDescriptionDialogComponent extends BaseComponent {

	public isDialog: boolean = false;
	public formGroup: UntypedFormGroup;
	public sections: DmpBlueprintDefinitionSection[] = [];

	dmpAutoCompleteConfiguration: SingleAutoCompleteConfiguration = {
		initialItems: (data?: any) => this.dmpService.query(this.buildAutocompleteLookup()).pipe(map(x => x.items)),
		filterFn: (searchQuery: string, data?: any) => this.dmpService.query(this.buildAutocompleteLookup(searchQuery)).pipe(map(x => x.items)),
		getSelectedItem: (selectedItem: any) => this.dmpService.query(this.buildAutocompleteLookup(null, null, [selectedItem])).pipe(map(x => x.items[0])),
		displayFn: (item: Dmp) => item.label,
		subtitleFn: (item: Dmp) => `${this.language.instant('DASHBOARD.ADD-NEW-DATASET.OPTIONS.CREATED-AT')} ${this.datePipe.transform(item.createdAt ,'dd/MM/yyyy')}`,
		titleFn: (item: Dmp) => item.label,
		valueAssign: (item: Dmp) => item.id,
	};

	private buildAutocompleteLookup(like?: string, excludedIds?: Guid[], ids?: Guid[], statuses?: DmpStatus[], dmpDescriptionTemplateSubQuery?: DmpDescriptionTemplateLookup): DmpLookup {
		const lookup: DmpLookup = new DmpLookup();
		lookup.page = { size: 100, offset: 0 };
		if (excludedIds && excludedIds.length > 0) { lookup.excludedIds = excludedIds; }
		if (ids && ids.length > 0) { lookup.ids = ids; }
		lookup.isActive = [IsActive.Active];
		lookup.statuses = [DmpStatus.Draft];
		lookup.project = {
			fields: [
				nameof<Dmp>(x => x.id),
				nameof<Dmp>(x => x.label),
				nameof<Dmp>(x => x.createdAt),
			]
		};
		if (dmpDescriptionTemplateSubQuery != null) lookup.dmpDescriptionTemplateSubQuery = dmpDescriptionTemplateSubQuery;
		lookup.order = { items: [nameof<Dmp>(x => x.label)] };
		if (like) { lookup.like = this.filterService.transformLike(like); }
		return lookup;
	}

	constructor(
		public dialogRef: MatDialogRef<StartNewDescriptionDialogComponent>,
		@Inject(MAT_DIALOG_DATA) public data: any,
		public dialog: MatDialog,
		private language: TranslateService,
		private dmpService: DmpService,
		private filterService: FilterService,
		private datePipe: DatePipe
	) {
		super();
		this.formGroup = data.formGroup;
	}

	cancel() {
		this.dialogRef.close();
	}

	send() {
		this.dialogRef.close(this.data);
	}

	close() {
		this.dialogRef.close(false);
	}

	next() {
		this.dialogRef.close(this.data);
	}

	startNewDmp() {
		this.data.startNewDmp = true;
		this.dialogRef.close(this.data);
	}

	getDatasetDisplay(item: any): string {
		// if (!this.isPublic) {
		// 	return (item['status'] ? this.language.instant('TYPES.DATASET-STATUS.FINALISED').toUpperCase() : this.language.instant('TYPES.DATASET-STATUS.DRAFT').toUpperCase()) + ': ' + item['label'];
		// }
		// else { return item['label']; }
		return item['label'] ? item['label'] : null;
	}
}
