import { HttpClient } from '@angular/common/http';
import { Component, Inject } from '@angular/core';
import { MAT_DIALOG_DATA, MatDialog, MatDialogRef } from '@angular/material/dialog';
import { DescriptionTemplate } from '@app/core/model/description-template/description-template';
import { DmpService } from '@app/core/services/dmp/dmp.service';
import { MatomoService } from '@app/core/services/matomo/matomo-service';
import { MultipleAutoCompleteConfiguration } from '@app/library/auto-complete/multiple/multiple-auto-complete-configuration';
import { BaseComponent } from '@common/base/base.component';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';

@Component({
	selector: 'dmp-upload-dialog',
	templateUrl: './dmp-upload-dialog.component.html',
	styleUrls: ['./dmp-upload-dialog.component.scss']
})
export class DmpUploadDialogComponent extends BaseComponent {
	dmpTitle: string;
	dmpBlueprints: any[] = [];
	files: File[] = [];

	profilesAutoCompleteConfiguration: MultipleAutoCompleteConfiguration = {
		filterFn: this.filterProfiles.bind(this),
		initialItems: (excludedItems: any[]) => this.filterProfiles('').pipe(map(result => result.filter(resultItem => (excludedItems || []).map(x => x.id).indexOf(resultItem.id) === -1))),
		displayFn: (item) => item['label'],
		titleFn: (item) => item['label'],
		subtitleFn: (item) => item['description'],
		popupItemActionIcon: 'visibility'
	};


	constructor(
		public dialogRef: MatDialogRef<DmpUploadDialogComponent>,
		private _service: DmpService,
		private dialog: MatDialog,
		private httpClient: HttpClient,
		private matomoService: MatomoService,
		@Inject(MAT_DIALOG_DATA) public data: any,
	) {
		super();
	}

	ngOnInit() {
		this.matomoService.trackPageView('DMP Upload Dialog');
	}

	cancel() {
		this.data.success = false;
		this.dialogRef.close(this.data);
	}

	close() {
		this.dialogRef.close(false);
	}

	confirm() {
		this.data.success = true;
		this.data.dmpTitle = this.dmpTitle;
		this.data.dmpBlueprints = this.dmpBlueprints;
		this.dialogRef.close(this.data);
	}

	uploadFile(event) {
		const fileList: FileList = event.target.files
		this.data.fileList = fileList;
		if (this.data.fileList.length > 0) {
			this.dmpTitle = fileList.item(0).name;
		}
		if (this.files.length === 1) {
			this.files.splice(0, 1);
		}
		this.files.push(...event.target.files);
	}

	selectFile(event) {
		const fileList: FileList = event.addedFiles
		this.data.fileList = fileList;
		if (this.data.fileList.length > 0) {
			this.dmpTitle = fileList[0].name;
		}
		if (this.files.length === 1) {
			this.files.splice(0, 1);
		}
		this.files.push(...event.addedFiles);
	}

	onRemove(event) {
		this.files.splice(0, 1);
		this.dmpTitle = null;
	}

	filterProfiles(value: string): Observable<DescriptionTemplate[]> {
		// TODO refactor
		// const request = new DataTableRequest<DatasetProfileCriteria>(null, null, { fields: ['+label'] });
		// const criteria = new DatasetProfileCriteria();
		// criteria.like = value;
		// request.criteria = criteria;
		// return this._service.searchDmpBlueprints(request);
		return null;
	}

	hasFile(): boolean {
		return this.files && this.files.length > 0;
	}
}
