import { Component, EventEmitter, OnInit, Output } from "@angular/core";
import { MatButtonToggleChange } from "@angular/material/button-toggle";
import { Router } from "@angular/router";
import { AuthService } from "@app/core/services/auth/auth.service";
import { PrincipalService } from "@app/core/services/http/principal.service";
import { BaseComponent } from "@common/base/base.component";
import { BaseHttpParams } from "@common/http/base-http-params";
import { InterceptorType } from "@common/http/interceptors/interceptor-type";
import { KeycloakService } from "keycloak-angular";
import { Observable, from } from "rxjs";
import { takeUntil } from "rxjs/operators";

@Component({
  selector: 'app-tenant-switch',
  templateUrl: 'tenant-switch.component.html',
  styleUrls: ['tenant-switch.component.scss']
})
export class TenantSwitchComponent extends BaseComponent implements OnInit {
	tenants: Observable<Array<string>>;

	constructor(
		private router: Router,
    private keycloakService: KeycloakService,
    private principalService: PrincipalService,
		private authService: AuthService,
	) {
		super();
	}
  
  get currentTenant(): string {
    return this.authService.selectedTenant();
  }

	ngOnInit() {
		this.tenants = this.loadUserTenants(); //TODO
		//this.tenantChange.emit(this.getCurrentLanguage())
	}

  loadUserTenants(): Observable<Array<string>> {
		const params = new BaseHttpParams();
		params.interceptorContext = {
			excludedInterceptors: [InterceptorType.TenantHeaderInterceptor]
		};
		return this.principalService.myTenants({ params: params });
	}

	onTenantSelected(selectedTenant: MatButtonToggleChange) {
    if (selectedTenant.value === undefined || selectedTenant.value === '') return;

    this.formSubmit(selectedTenant.value);
		this.loadUser();
	}

	formSubmit(selectedTenant: string): void {
		this.authService.selectedTenant(selectedTenant);
	}

	loadUser(): void {
		this.authService.prepareAuthRequest(from(this.keycloakService.getToken()), {})
      .pipe(takeUntil(this._destroyed))
      .subscribe(
        () => {
          this.authService.onAuthenticateSuccessReload();
        }, 
        (error) => this.authService.onAuthenticateError(error)
      );
	}
}