package eu.old.eudat.configurations.dynamicgrant;

import eu.old.eudat.configurations.dynamicgrant.entities.Configuration;
import eu.old.eudat.configurations.dynamicgrant.entities.Property;
import eu.old.eudat.models.data.dynamicfields.Dependency;
import eu.old.eudat.models.data.dynamicfields.DynamicField;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Profile;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;

import jakarta.xml.bind.JAXBContext;
import jakarta.xml.bind.Unmarshaller;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.LinkedList;
import java.util.List;

/**
 * Created by ikalyvas on 5/4/2018.
 */

@Service("dynamicGrantConfiguration")
@Profile("devel")
public class DynamicGrantConfigurationDevelImpl implements DynamicGrantConfiguration {
    private static final Logger logger = LoggerFactory.getLogger(DynamicGrantConfigurationDevelImpl.class);

    private Configuration configuration;

    private List<DynamicField> fields;

    private Environment environment;

    @Autowired
    public DynamicGrantConfigurationDevelImpl(Environment environment) {
        this.environment = environment;
    }

    @Override
    public Configuration getConfiguration() {
        if (this.configuration != null) return this.configuration;
        String fileUrl = this.environment.getProperty("configuration.dynamicGrantUrl");
        logger.info("Loaded also config file: " + fileUrl);
        String current = null;
        InputStream is = null;
        try {
            current = new java.io.File(".").getCanonicalPath();

            JAXBContext jaxbContext = JAXBContext.newInstance(Configuration.class);
            Unmarshaller jaxbUnmarshaller = jaxbContext.createUnmarshaller();
            is = new URL("file:///"+ current + "/web/src/main/resources/GrantConfiguration.xml").openStream();
            this.configuration = (Configuration) jaxbUnmarshaller.unmarshal(is);
        } catch (Exception ex) {
            logger.error("Cannot find in folder" + current, ex);
        } finally {
            try {
                if (is != null) is.close();
            } catch (IOException e) {
                logger.warn("Warning: Could not close a stream after reading from file: " + fileUrl, e);
            }
        }
        return this.configuration;
    }

    @Override
    public List<DynamicField> getFields() {
        if (this.fields != null) return this.fields;
        Configuration configuration = this.getConfiguration();
        List<DynamicField> fields = new LinkedList<>();
        List<Property> properties = configuration.getConfigurationProperties();
        properties.stream().forEach(item -> {
            DynamicField dynamicField = new DynamicField();
            dynamicField.setId(item.getId());
            dynamicField.setName(item.getName());
            dynamicField.setQueryProperty(item.getQueryProperty());
            dynamicField.setRequired(item.getRequired());
            List<Dependency> dependencies = new LinkedList<>();
            item.getDependencies().stream().forEach(dependency -> {
                Dependency modelDependency = new Dependency();
                modelDependency.setId(dependency.getId());
                modelDependency.setQueryProperty(dependency.getQueryProperty());
                dependencies.add(modelDependency);
            });
            dynamicField.setDependencies(dependencies);
            fields.add(dynamicField);
        });
        this.fields = fields;
        return fields;
    }
}
