package eu.old.eudat.controllers;

import eu.old.eudat.data.dao.criteria.RequestItem;
import eu.old.eudat.data.entities.DescriptionTemplate;
import eu.old.eudat.logic.managers.AdminManager;
import eu.old.eudat.logic.managers.DatasetProfileManager;
import eu.old.eudat.logic.security.claims.ClaimedAuthorities;
import eu.old.eudat.logic.services.ApiContext;
import eu.old.eudat.models.data.components.commons.datafield.AutoCompleteData;
import eu.old.eudat.models.data.externaldataset.ExternalAutocompleteFieldModel;
import eu.old.eudat.models.data.helpers.common.AutoCompleteLookupItem;
import eu.old.eudat.models.data.helpers.common.AutoCompleteOptionsLookupItem;
import eu.old.eudat.models.data.helpers.responses.ResponseItem;
import eu.old.eudat.models.data.security.Principal;
import eu.old.eudat.models.data.admin.composite.DatasetProfile;
import eu.old.eudat.models.data.entities.xmlmodels.datasetprofiledefinition.Field;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.xml.xpath.XPathExpressionException;
import java.util.List;
import java.util.UUID;

import static eu.old.eudat.types.Authorities.ADMIN;

@RestController
@CrossOrigin
@RequestMapping(value = {"/api"})
public class DatasetProfileController extends BaseController {

    private DatasetProfileManager datasetProfileManager;

    @Autowired
    public DatasetProfileController(ApiContext apiContext, DatasetProfileManager datasetProfileManager) {
        super(apiContext);
        this.datasetProfileManager = datasetProfileManager;
    }

/*    @Transactional
    @RequestMapping(method = RequestMethod.POST, value = {"/datasetprofile/save/{id}"}, consumes = "application/json", produces = "application/json")
    public ResponseEntity<Object> updateDataset(@PathVariable String id, @RequestBody PropertiesModel properties) {
        eu.old.entities.data.eudat.Dataset dataset = this.getApiContext().getOperationsContext().getDatabaseRepository().getDatasetDao().find(UUID.fromString(id));
        Map<String, Object> values = new HashMap<>();
        properties.toMap(values);
        JSONObject jobject = new JSONObject(values);
        dataset.setProperties(jobject.toString());
        dataset.setStatus((short) properties.getStatus());
        this.getApiContext().getOperationsContext().getDatabaseRepository().getDatasetDao().createOrUpdate(dataset);  //TODO
        return ResponseEntity.status(HttpStatus.OK).body(properties);
    }*/

    @Transactional
    @RequestMapping(method = RequestMethod.POST, value = {"/datasetprofile/clone/{id}"}, consumes = "application/json", produces = "application/json")
    public ResponseEntity<ResponseItem<DatasetProfile>> clone(@PathVariable String id, @ClaimedAuthorities(claims = {ADMIN}) Principal principal) {
        DescriptionTemplate profile = this.datasetProfileManager.clone(id);
        DatasetProfile datasetprofile = AdminManager.generateDatasetProfileModel(profile);
        datasetprofile.setLabel(profile.getLabel() + " new ");
        return ResponseEntity.status(HttpStatus.OK).body(new ResponseItem<DatasetProfile>().payload(datasetprofile));
    }

    @RequestMapping(method = RequestMethod.POST, value = {"/search/autocomplete"}, consumes = "application/json", produces = "application/json")
    public ResponseEntity<Object> getDataForAutocomplete(@RequestBody RequestItem<AutoCompleteLookupItem> lookupItem) throws XPathExpressionException {
        DescriptionTemplate descriptionTemplate = this.getApiContext().getOperationsContext().getDatabaseRepository().getDatasetProfileDao().find(UUID.fromString(lookupItem.getCriteria().getProfileID()));
        Field modelfield = this.datasetProfileManager.queryForField(descriptionTemplate.getDefinition(), lookupItem.getCriteria().getFieldID());
        AutoCompleteData data = (AutoCompleteData) modelfield.getData();
        List<ExternalAutocompleteFieldModel> items = this.datasetProfileManager.getAutocomplete(data, lookupItem.getCriteria().getLike());
        return ResponseEntity.status(HttpStatus.OK).body(items);
    }

    @RequestMapping(method = RequestMethod.POST, value = {"/search/autocompleteOptions"}, consumes = "application/json", produces = "application/json")
    public ResponseEntity<Object> getDataForAutocompleteOptions(@RequestBody RequestItem<AutoCompleteOptionsLookupItem> lookupItem) {
        AutoCompleteData data = new AutoCompleteData();
        data.setAutoCompleteSingleDataList(lookupItem.getCriteria().getAutoCompleteSingleDataList());
        List<ExternalAutocompleteFieldModel> items = this.datasetProfileManager.getAutocomplete(data, lookupItem.getCriteria().getLike());
        return ResponseEntity.status(HttpStatus.OK).body(items);
    }

}
