package eu.old.eudat.logic.managers;

import eu.old.eudat.data.dao.criteria.DatasetCriteria;
import eu.old.eudat.data.dao.entities.DatasetDao;
import eu.old.eudat.data.dao.entities.DatasetProfileDao;
import eu.old.eudat.data.entities.DescriptionTemplate;
import eu.old.eudat.exceptions.datasetprofile.DatasetProfileWithDatasetsExeption;
import eu.old.eudat.logic.builders.entity.DatasetProfileBuilder;
import eu.old.eudat.models.data.entities.xmlmodels.datasetprofiledefinition.ViewStyleModel;
import eu.old.eudat.models.data.admin.composite.DatasetProfile;
import eu.old.eudat.logic.services.ApiContext;
import eu.old.eudat.logic.utilities.builders.ModelBuilder;
import eu.old.eudat.logic.utilities.builders.XmlBuilder;
import eu.old.eudat.models.data.entities.xmlmodels.datasetprofiledefinition.Page;
import eu.old.eudat.models.data.entities.xmlmodels.datasetprofiledefinition.Section;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.util.Date;
import java.util.UUID;

public class AdminManager {

    public static DescriptionTemplate generateViewStyleDefinition(DatasetProfile profile, ApiContext apiContext) {
        ViewStyleModel viewStyleModel = new ViewStyleModel();
        viewStyleModel.setSections(new ModelBuilder().toViewStyleDefinition(profile.getSections(), Section.class));
        viewStyleModel.setPages(new ModelBuilder().toViewStyleDefinition(profile.getPages(), Page.class));
        Document viewStyleDoc = XmlBuilder.getDocument();
        Element elementViewStyle = viewStyleModel.toXml(viewStyleDoc);
        viewStyleDoc.appendChild(elementViewStyle);
        String xml = XmlBuilder.generateXml(viewStyleDoc);

        if (profile.getDescription() == null) {
            profile.setDescription("");
        }

        if (profile.getLanguage() == null) {
            profile.setLanguage("en");
        }

        DescriptionTemplate descriptionTemplate = apiContext.getOperationsContext().getBuilderFactory().getBuilder(DatasetProfileBuilder.class).definition(xml).label(profile.getLabel())
                .status(profile.getStatus()).created(new Date()).description(profile.getDescription()).language(profile.getLanguage())
                .build();

        if (descriptionTemplate.getGroupId() == null) {
            descriptionTemplate.setGroupId(UUID.randomUUID());
        }

        if (descriptionTemplate.getVersion() == null) {
            descriptionTemplate.setVersion((short)1);
        }

        return descriptionTemplate;
    }

    public static DatasetProfile generateDatasetProfileModel(DescriptionTemplate profile) {
        Document viewStyleDoc = XmlBuilder.fromXml(profile.getDefinition());
        Element root = viewStyleDoc.getDocumentElement();
        ViewStyleModel viewstyle = new ViewStyleModel().fromXml(root);

        DatasetProfile datasetprofile = new DatasetProfile();
        datasetprofile.buildProfile(viewstyle);
        return datasetprofile;
    }


    public static DescriptionTemplate inactivate(DatasetProfileDao datasetProfileRepository, DatasetDao datasetDao, String id) {
        DatasetCriteria datasetsForThatDatasetProfile = new DatasetCriteria();
        datasetsForThatDatasetProfile.setProfileDatasetId(UUID.fromString(id));
        if (datasetDao.getWithCriteria(datasetsForThatDatasetProfile).count() == 0) {
            DescriptionTemplate detasetProfile = datasetProfileRepository.find(UUID.fromString(id));
            detasetProfile.setStatus(DescriptionTemplate.Status.DELETED.getValue());
            detasetProfile = datasetProfileRepository.createOrUpdate(detasetProfile);
            return detasetProfile;
        } else {
            throw new DatasetProfileWithDatasetsExeption("This profile can not deleted, because Datasets are associated with it");
        }
    }
}