package eu.old.eudat.migration;

import eu.eudat.commons.XmlHandlingService;
import eu.eudat.commons.enums.IsActive;
import eu.eudat.commons.enums.ReferenceFieldDataType;
import eu.eudat.commons.enums.ReferenceSourceType;
import eu.eudat.commons.types.reference.DefinitionEntity;
import eu.eudat.commons.types.reference.FieldEntity;
import eu.eudat.convention.ConventionService;
import eu.eudat.data.ReferenceEntity;
import eu.old.eudat.data.dao.entities.OrganisationDao;
import eu.old.eudat.data.entities.Organisation;
import eu.old.eudat.logic.services.operations.DatabaseRepository;
import gr.cite.tools.logging.LoggerService;
import jakarta.persistence.EntityManager;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Locale;

@Service
public class OrganizationMigrationService {
	private static final LoggerService logger = new LoggerService(LoggerFactory.getLogger(OrganizationMigrationService.class));

	private static final int PageSize = 500;
	private static final String InternalReferenceSource = "Internal";
	private final DatabaseRepository databaseRepository;
	private final EntityManager entityManager;
	private final ConventionService conventionService;
	private final XmlHandlingService xmlHandlingService;
	public OrganizationMigrationService(DatabaseRepository databaseRepository, EntityManager entityManager, ConventionService conventionService, XmlHandlingService xmlHandlingService) {
		this.databaseRepository = databaseRepository;
		this.entityManager = entityManager;
		this.conventionService = conventionService;
		this.xmlHandlingService = xmlHandlingService;
	}
	
	public void migrate(){
		OrganisationDao organisationDao = databaseRepository.getOrganisationDao();
		long total = organisationDao.asQueryable().count();
		logger.debug("Migrate Organisation Total : " + total);
		int page = 0;

		List<Organisation> items;
		do {
			items = organisationDao.asQueryable().orderBy((builder, root) -> builder.asc(root.get("created"))).orderBy((builder, root) -> builder.asc(root.get("ID"))).skip(page * PageSize).take(PageSize).toList();
			if (items != null && !items.isEmpty()) {
				logger.debug("Migrate Organisation " + page * PageSize + " of " + total);
				for (Organisation item : items) {
					entityManager.detach(item);
					if (item.getReference() == null || !item.getReference().contains(":")){
						logger.warn("Reference generated because is null Organisation " + item.getId());
						item.setReference(InternalReferenceSource + ":" + item.getId().toString().replace("-", "").toLowerCase(Locale.ROOT));
					}
					
					eu.old.eudat.models.data.dmp.Organisation model = new eu.old.eudat.models.data.dmp.Organisation().fromDataModel(item);

					String[] referenceParts = item.getReference().split(":", 2);
					boolean isInternal = referenceParts[0].equals(InternalReferenceSource);
					
					ReferenceEntity data = new ReferenceEntity();
					data.setId(item.getId());
					data.setLabel(item.getLabel());
					data.setAbbreviation(item.getAbbreviation());
					data.setIsActive(IsActive.Active);
					data.setTypeId(ReferenceTypeIds.Organizations);
					data.setCreatedAt(item.getCreated().toInstant());
					data.setUpdatedAt(item.getModified().toInstant());
					data.setReference(referenceParts[1]);
					data.setAbbreviation(item.getAbbreviation());
					data.setSource(isInternal? "Internal" : referenceParts[0]);
					data.setSourceType(isInternal? ReferenceSourceType.Internal : ReferenceSourceType.External);
					if (!this.conventionService.isNullOrEmpty(item.getUri())){
						DefinitionEntity definitionEntity = new DefinitionEntity();
						FieldEntity fieldEntity = new FieldEntity();
						fieldEntity.setCode("uri");
						fieldEntity.setDataType(ReferenceFieldDataType.Text);
						fieldEntity.setValue(item.getUri());
						definitionEntity.setFields(List.of(fieldEntity));
						data.setDefinition(this.xmlHandlingService.toXmlSafe(definitionEntity));
					}
					this.entityManager.persist(data);
				}
				this.entityManager.flush();
				
				page++;
			}
		} while (items != null && !items.isEmpty());
	}
}
