package eu.old.eudat.migration;

import eu.eudat.commons.XmlHandlingService;
import eu.eudat.commons.enums.IsActive;
import eu.eudat.commons.enums.ReferenceFieldDataType;
import eu.eudat.commons.enums.ReferenceSourceType;
import eu.eudat.commons.types.reference.DefinitionEntity;
import eu.eudat.commons.types.reference.FieldEntity;
import eu.eudat.convention.ConventionService;
import eu.eudat.data.ReferenceEntity;
import eu.old.eudat.data.dao.entities.ResearcherDao;
import eu.old.eudat.data.entities.Researcher;
import eu.old.eudat.logic.services.operations.DatabaseRepository;
import gr.cite.tools.logging.LoggerService;
import jakarta.persistence.EntityManager;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

@Service
public class ResearcherMigrationService {
	private static final LoggerService logger = new LoggerService(LoggerFactory.getLogger(ResearcherMigrationService.class));

	private static final int PageSize = 500;
	private static final String InternalReferenceSource = "dmp";
	private final DatabaseRepository databaseRepository;
	private final EntityManager entityManager;
	private final ConventionService conventionService;
	private final XmlHandlingService xmlHandlingService;
	public ResearcherMigrationService(DatabaseRepository databaseRepository, EntityManager entityManager, ConventionService conventionService, XmlHandlingService xmlHandlingService) {
		this.databaseRepository = databaseRepository;
		this.entityManager = entityManager;
		this.conventionService = conventionService;
		this.xmlHandlingService = xmlHandlingService;
	}
	
	public void migrate(){
		ResearcherDao researcherDao = databaseRepository.getResearcherDao();
		long total = researcherDao.asQueryable().count();
		logger.debug("Migrate Researcher Total : " + total);
		int page = 0;

		List<Researcher> items;
		do {
			items = researcherDao.asQueryable().orderBy((builder, root) -> builder.asc(root.get("created"))).orderBy((builder, root) -> builder.asc(root.get("ID"))).skip(page * PageSize).take(PageSize).toList();
			if (items != null && !items.isEmpty()) {
				logger.debug("Migrate Researcher " + page * PageSize + " of " + total);
				for (Researcher item : items) {
					entityManager.detach(item);
					if (item.getReference() == null || !item.getReference().contains(":")){
						logger.warn("Reference generated because is null Researcher " + item.getId());
						item.setReference(InternalReferenceSource + ":" + item.getId().toString().replace("-", "").toLowerCase(Locale.ROOT));
					}
					eu.old.eudat.models.data.dmp.Researcher model = new eu.old.eudat.models.data.dmp.Researcher().fromDataModel(item);

					String[] referenceParts = item.getReference().split(":", 2);
					boolean isInternal = referenceParts[0].equals(InternalReferenceSource);
					
					ReferenceEntity data = new ReferenceEntity();
					data.setId(item.getId());
					data.setLabel(item.getLabel());
					data.setIsActive(IsActive.Active);
					data.setTypeId(ReferenceTypeIds.Researcher);
					data.setCreatedAt(item.getCreated().toInstant());
					data.setUpdatedAt(item.getModified().toInstant());
					if (item.getCreationUser() != null) data.setCreatedById(item.getCreationUser().getId());
					data.setReference(referenceParts[1]);
					data.setSource(isInternal? "Internal" : referenceParts[0]);
					data.setSourceType(isInternal? ReferenceSourceType.Internal : ReferenceSourceType.External);

					DefinitionEntity definitionEntity = new DefinitionEntity();
					definitionEntity.setFields(new ArrayList<>());
					if (!this.conventionService.isNullOrEmpty(item.getUri())){
						FieldEntity fieldEntity = new FieldEntity();
						fieldEntity.setCode("uri");
						fieldEntity.setDataType(ReferenceFieldDataType.Text);
						fieldEntity.setValue(item.getUri());
						definitionEntity.getFields().add(fieldEntity);
					}
					if (!this.conventionService.isNullOrEmpty(item.getPrimaryEmail())){
						FieldEntity fieldEntity = new FieldEntity();
						fieldEntity.setCode("primaryEmail");
						fieldEntity.setDataType(ReferenceFieldDataType.Text);
						fieldEntity.setValue(item.getPrimaryEmail());
						definitionEntity.getFields().add(fieldEntity);
					}


					if (!definitionEntity.getFields().isEmpty()){
						data.setDefinition(this.xmlHandlingService.toXmlSafe(definitionEntity));
					}
					
					this.entityManager.persist(data);
				}
				this.entityManager.flush();
				
				page++;
			}
		} while (items != null && !items.isEmpty());
	}
}
