package gr.cite.notification.integrationevent.inbox.notify;

import gr.cite.notification.common.StringUtils;
import gr.cite.notification.common.enums.IsActive;
import gr.cite.notification.common.enums.NotificationContactType;
import gr.cite.notification.integrationevent.inbox.ConsistencyHandler;
import gr.cite.notification.model.User;
import gr.cite.notification.model.builder.UserBuilder;
import gr.cite.notification.query.UserQuery;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.data.query.QueryFactory;
import gr.cite.tools.fieldset.BaseFieldSet;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

@Component
@Scope(value = ConfigurableBeanFactory.SCOPE_PROTOTYPE)
public class NotifyConsistencyHandler implements ConsistencyHandler<NotifyConsistencyPredicates> {

    private final QueryFactory queryFactory;

    private final BuilderFactory builderFactory;

    public NotifyConsistencyHandler(QueryFactory queryFactory, BuilderFactory builderFactory) {
        this.queryFactory = queryFactory;
        this.builderFactory = builderFactory;
    }

    @Override
    public Boolean isConsistent(NotifyConsistencyPredicates consistencyPredicates) {
        if (consistencyPredicates.getUserId() != null) {
            UserQuery query = this.queryFactory.query(UserQuery.class).ids(consistencyPredicates.getUserId()).isActive(IsActive.Active);

            BaseFieldSet fieldSet = new BaseFieldSet(
                    User._id,
                    User._isActive
            );
            User user = this.builderFactory.builder(UserBuilder.class).build(fieldSet, query.firstAs(fieldSet));
            if (user == null)
                return false;
            return consistencyPredicates.getContactTypeHint() == null || consistencyPredicates.getContactTypeHint() == NotificationContactType.IN_APP || !StringUtils.isNullOrEmpty(consistencyPredicates.getContactHint());
        }
        return true;
    }

}
