package gr.cite.notification.model.builder.notificationtemplate;

import gr.cite.notification.authorization.AuthorizationFlags;
import gr.cite.notification.common.types.notification.FieldInfo;
import gr.cite.notification.common.types.notificationtemplate.FieldInfoEntity;
import gr.cite.notification.convention.ConventionService;
import gr.cite.notification.model.builder.BaseBuilder;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.exception.MyApplicationException;
import gr.cite.tools.fieldset.FieldSet;
import gr.cite.tools.logging.DataLogEntry;
import gr.cite.tools.logging.LoggerService;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.*;

@Component
@Scope(value = ConfigurableBeanFactory.SCOPE_PROTOTYPE)
public class FieldInfoBuilder extends BaseBuilder<FieldInfo, FieldInfoEntity> {

    private EnumSet<AuthorizationFlags> authorize = EnumSet.of(AuthorizationFlags.None);

    @Autowired
    public FieldInfoBuilder(
            ConventionService conventionService, BuilderFactory builderFactory) {
        super(conventionService, new LoggerService(LoggerFactory.getLogger(FieldInfoBuilder.class)));
    }

    public FieldInfoBuilder authorize(EnumSet<AuthorizationFlags> values) {
        this.authorize = values;
        return this;
    }

    @Override
    public List<FieldInfo> build(FieldSet fields, List<FieldInfoEntity> data) throws MyApplicationException {
        this.logger.debug("building for {} items requesting {} fields", Optional.ofNullable(data).map(List::size).orElse(0), Optional.ofNullable(fields).map(FieldSet::getFields).map(Set::size).orElse(0));
        this.logger.trace(new DataLogEntry("requested fields", fields));
        if (fields == null || data == null || fields.isEmpty())
            return new ArrayList<>();

        List<FieldInfo> models = new ArrayList<>();
        for (FieldInfoEntity d : data) {
            FieldInfo m = new FieldInfo();
            if (fields.hasField(this.asIndexer(FieldInfo._key))) m.setKey(d.getKey());
            if (fields.hasField(this.asIndexer(FieldInfo._value))) m.setValue(d.getValue());
            if (fields.hasField(this.asIndexer(FieldInfo._type))) m.setType(d.getType());

            models.add(m);
        }
        this.logger.debug("build {} items", Optional.of(models).map(List::size).orElse(0));
        return models;
    }
}
