package gr.cite.notification.model.builder.notificationtemplate;

import gr.cite.notification.authorization.AuthorizationFlags;
import gr.cite.notification.common.types.notificationtemplate.FieldOptionsEntity;
import gr.cite.notification.convention.ConventionService;
import gr.cite.notification.model.builder.BaseBuilder;
import gr.cite.notification.model.notificationtemplate.FieldOptions;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.exception.MyApplicationException;
import gr.cite.tools.fieldset.FieldSet;
import gr.cite.tools.logging.DataLogEntry;
import gr.cite.tools.logging.LoggerService;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.*;

@Component
@Scope(value = ConfigurableBeanFactory.SCOPE_PROTOTYPE)
public class FieldOptionsBuilder extends BaseBuilder<FieldOptions, FieldOptionsEntity> {

    private final BuilderFactory builderFactory;
    private EnumSet<AuthorizationFlags> authorize = EnumSet.of(AuthorizationFlags.None);

    @Autowired
    public FieldOptionsBuilder(
            ConventionService conventionService, BuilderFactory builderFactory) {
        super(conventionService, new LoggerService(LoggerFactory.getLogger(FieldOptionsBuilder.class)));
        this.builderFactory = builderFactory;
    }

    public FieldOptionsBuilder authorize(EnumSet<AuthorizationFlags> values) {
        this.authorize = values;
        return this;
    }

    @Override
    public List<FieldOptions> build(FieldSet fields, List<FieldOptionsEntity> data) throws MyApplicationException {
        this.logger.debug("building for {} items requesting {} fields", Optional.ofNullable(data).map(List::size).orElse(0), Optional.ofNullable(fields).map(FieldSet::getFields).map(Set::size).orElse(0));
        this.logger.trace(new DataLogEntry("requested fields", fields));
        if (fields == null || data == null || fields.isEmpty())
            return new ArrayList<>();

        FieldSet optionalFields = fields.extractPrefixed(this.asPrefix(FieldOptions._optional));

        List<FieldOptions> models = new ArrayList<>();
        for (FieldOptionsEntity d : data) {
            FieldOptions m = new FieldOptions();
            if (fields.hasField(this.asIndexer(FieldOptions._mandatory))) m.setMandatory(d.getMandatory());
            if (!optionalFields.isEmpty() && d.getOptional() != null) {
                m.setOptional(this.builderFactory.builder(FieldInfoBuilder.class).authorize(this.authorize).build(optionalFields, d.getOptional()));
            }
            if (fields.hasField(this.asIndexer(FieldOptions._formatting))) m.setFormatting(d.getFormatting());


            models.add(m);
        }
        this.logger.debug("build {} items", Optional.of(models).map(List::size).orElse(0));
        return models;
    }
}
