package gr.cite.notification.model.persist;

import gr.cite.notification.common.enums.NotificationContactType;
import gr.cite.notification.common.validation.BaseValidator;
import gr.cite.notification.convention.ConventionService;
import gr.cite.notification.errorcode.ErrorThesaurusProperties;
import gr.cite.tools.validation.specification.Specification;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.UUID;

public class UserNotificationPreferencePersist {

    private UUID userId;
    public static final String _userId = "userId";
    private Map<UUID, List<NotificationContactType>> notificationPreferences;
    public static final String _notificationPreferences = "notificationPreferences";

    public UUID getUserId() {
        return userId;
    }

    public void setUserId(UUID userId) {
        this.userId = userId;
    }

    public Map<UUID, List<NotificationContactType>> getNotificationPreferences() {
        return notificationPreferences;
    }

    public void setNotificationPreferences(Map<UUID, List<NotificationContactType>> notificationPreferences) {
        this.notificationPreferences = notificationPreferences;
    }

    @Component(UserNotificationPreferencePersist.UserNotificationPreferencePersistValidator.ValidatorName)
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public static class UserNotificationPreferencePersistValidator extends BaseValidator<UserNotificationPreferencePersist> {

        public static final String ValidatorName = "UserNotificationPreferencePersistValidator";

        private final MessageSource messageSource;

        protected UserNotificationPreferencePersistValidator(ConventionService conventionService, ErrorThesaurusProperties errors, MessageSource messageSource) {
            super(conventionService, errors);
            this.messageSource = messageSource;
        }

        @Override
        protected Class<UserNotificationPreferencePersist> modelClass() {
            return UserNotificationPreferencePersist.class;
        }

        @Override
        protected List<Specification> specifications(UserNotificationPreferencePersist item) {
            return Arrays.asList(
                    this.spec()
                            .must(() -> !this.isNull(item.getUserId()))
                            .failOn(UserNotificationPreferencePersist._userId).failWith(messageSource.getMessage("Validation_Required", new Object[]{UserNotificationPreferencePersist._userId}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .must(() -> !this.isNull(item.getNotificationPreferences()))
                            .failOn(UserNotificationPreferencePersist._notificationPreferences).failWith(messageSource.getMessage("Validation_Required", new Object[]{UserNotificationPreferencePersist._notificationPreferences}, LocaleContextHolder.getLocale()))
            );
        }
    }
}
