package gr.cite.notification.model.persist.notificationtemplate;

import gr.cite.notification.common.enums.NotificationDataType;
import gr.cite.notification.common.validation.BaseValidator;
import gr.cite.notification.convention.ConventionService;
import gr.cite.notification.errorcode.ErrorThesaurusProperties;
import gr.cite.tools.validation.ValidatorFactory;
import gr.cite.tools.validation.specification.Specification;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;

public class FieldInfoPersist {

	private String key;
	public static final String _key = "key";

	private NotificationDataType type;
	public static final String _type = "type";

	private String value;
	public static final String _value = "value";

	public FieldInfoPersist(String key, NotificationDataType type, String value) {
		this.key = key;
		this.type = type;
		this.value = value;
	}

	public FieldInfoPersist() {
	}

	public String getKey() {
		return key;
	}

	public void setKey(String key) {
		this.key = key;
	}

	public NotificationDataType getType() {
		return type;
	}

	public void setType(NotificationDataType type) {
		this.type = type;
	}

	public String getValue() {
		return value;
	}

	public void setValue(String value) {
		this.value = value;
	}

	@Component(FieldInfoPersist.FieldInfoPersistValidator.ValidatorName)
	@Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
	public static class FieldInfoPersistValidator extends BaseValidator<FieldInfoPersist> {

		public static final String ValidatorName = " FieldInfoPersistValidator";

		private final MessageSource messageSource;

		protected FieldInfoPersistValidator(ConventionService conventionService, ErrorThesaurusProperties errors, MessageSource messageSource) {
			super(conventionService, errors);
			this.messageSource = messageSource;
		}

		@Override
		protected Class<FieldInfoPersist> modelClass() {
			return FieldInfoPersist.class;
		}

		@Override
		protected List<Specification> specifications(FieldInfoPersist item) {
			return Arrays.asList(
					this.spec()
							.must(() -> !this.isEmpty(item.getKey()))
							.failOn(FieldInfoPersist._key).failWith(messageSource.getMessage("Validation_Required", new Object[]{FieldInfoPersist._key}, LocaleContextHolder.getLocale())),
					this.spec()
							.must(() -> !this.isNull(item.getType()))
							.failOn(FieldInfoPersist._type).failWith(messageSource.getMessage("Validation_Required", new Object[]{FieldInfoPersist._type}, LocaleContextHolder.getLocale())),
					this.spec()
							.must(() -> !this.isEmpty(item.getValue()))
							.failOn(FieldInfoPersist._value).failWith(messageSource.getMessage("Validation_Required", new Object[]{FieldInfoPersist._value}, LocaleContextHolder.getLocale()))
			);
		}
	}
}
