package gr.cite.notification.model.persist.notificationtemplate;

import gr.cite.notification.common.validation.BaseValidator;
import gr.cite.notification.convention.ConventionService;
import gr.cite.notification.errorcode.ErrorThesaurusProperties;
import gr.cite.tools.validation.ValidatorFactory;
import gr.cite.tools.validation.specification.Specification;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

public class FieldOptionsPersist {

    private List<String> mandatory;
    public static final String _mandatory = "mandatory";

    private List<FieldInfoPersist> optional;
    public static final String _optional = "optional";

    private Map<String, String> formatting;
    public static final String _formatting = "formatting";

    public List<String> getMandatory() {
        return mandatory;
    }

    public void setMandatory(List<String> mandatory) {
        this.mandatory = mandatory;
    }

    public List<FieldInfoPersist> getOptional() {
        return optional;
    }

    public void setOptional(List<FieldInfoPersist> optional) {
        this.optional = optional;
    }

    public Map<String, String> getFormatting() {
        return formatting;
    }

    public void setFormatting(Map<String, String> formatting) {
        this.formatting = formatting;
    }


    @Component(FieldOptionsPersist.FieldOptionsPersistValidator.ValidatorName)
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public static class FieldOptionsPersistValidator extends BaseValidator<FieldOptionsPersist> {

        public static final String ValidatorName = " FieldOptionsPersistValidator";

        private final MessageSource messageSource;

        private final ValidatorFactory validatorFactory;

        protected FieldOptionsPersistValidator(ConventionService conventionService, ErrorThesaurusProperties errors, MessageSource messageSource, ValidatorFactory validatorFactory) {
            super(conventionService, errors);
            this.messageSource = messageSource;
            this.validatorFactory = validatorFactory;
        }

        @Override
        protected Class<FieldOptionsPersist> modelClass() {
            return FieldOptionsPersist.class;
        }

        @Override
        protected List<Specification> specifications(FieldOptionsPersist item) {
            return Arrays.asList(
                    this.navSpec()
                            .iff(() -> !this.isListNullOrEmpty(item.getOptional()))
                            .on(FieldOptionsPersist._optional)
                            .over(item.getOptional())
                            .using((itm) -> this.validatorFactory.validator(FieldInfoPersist.FieldInfoPersistValidator.class)),
                    this.spec()
                            .iff(() -> !this.isListNullOrEmpty(item.getMandatory()) || !this.isNull(item.getOptional()))
                            .must(() -> !this.isNull(item.getFormatting()))
                            .failOn(FieldOptionsPersist._formatting).failWith(messageSource.getMessage("Validation_Required", new Object[]{FieldOptionsPersist._formatting}, LocaleContextHolder.getLocale()))
            );

        }
    }
}
