package gr.cite.notification.model.persist.notificationtemplate;

import gr.cite.notification.common.enums.EmailOverrideMode;
import gr.cite.notification.common.validation.BaseValidator;
import gr.cite.notification.convention.ConventionService;
import gr.cite.notification.errorcode.ErrorThesaurusProperties;
import gr.cite.tools.validation.ValidatorFactory;
import gr.cite.tools.validation.specification.Specification;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;

public class NotificationTemplateValuePersist {

    private String subjectText;
    public static final String _subjectText = "subjectText";

    private String subjectKey;
    public static final String _subjectKey = "subjectKey";

    private FieldOptionsPersist subjectFieldOptions;
    public static final String _subjectFieldOptions = "subjectFieldOptions";

    private String bodyText;
    public static final String _bodyText = "bodyText";

    private String bodyKey;
    public static final String _bodyKey = "bodyKey";
    private String priorityKey;
    public static final String _priorityKey = "priorityKey";
    private Boolean allowAttachments = false;
    public static final String _allowAttachments = "allowAttachments";
    private List<String> cc;
    public static final String _cc = "cc";

    private EmailOverrideMode ccMode;
    public static final String _ccMode = "ccMode";
    private List<String> bcc;
    public static final String _bcc = "bcc";

    private EmailOverrideMode bccMode;
    public static final String _bccMode = "bccMode";
    private List<String> extraDataKeys;
    public static final String _extraDataKeys = "extraDataKeys";

    private FieldOptionsPersist bodyFieldOptions;
    public static final String _bodyFieldOptions = "bodyFieldOptions";

    public String getSubjectText() {
        return subjectText;
    }

    public void setSubjectText(String subjectText) {
        this.subjectText = subjectText;
    }

    public String getSubjectKey() {
        return subjectKey;
    }

    public void setSubjectKey(String subjectKey) {
        this.subjectKey = subjectKey;
    }

    public FieldOptionsPersist getSubjectFieldOptions() {
        return subjectFieldOptions;
    }

    public void setSubjectFieldOptions(FieldOptionsPersist subjectFieldOptions) {
        this.subjectFieldOptions = subjectFieldOptions;
    }

    public String getBodyText() {
        return bodyText;
    }

    public void setBodyText(String bodyText) {
        this.bodyText = bodyText;
    }

    public String getBodyKey() {
        return bodyKey;
    }

    public void setBodyKey(String bodyKey) {
        this.bodyKey = bodyKey;
    }

    public String getPriorityKey() {
        return priorityKey;
    }

    public void setPriorityKey(String priorityKey) {
        this.priorityKey = priorityKey;
    }

    public Boolean getAllowAttachments() {
        return allowAttachments;
    }

    public void setAllowAttachments(Boolean allowAttachments) {
        this.allowAttachments = allowAttachments;
    }

    public List<String> getCc() {
        return cc;
    }

    public void setCc(List<String> cc) {
        this.cc = cc;
    }

    public EmailOverrideMode getCcMode() {
        return ccMode;
    }

    public void setCcMode(EmailOverrideMode ccMode) {
        this.ccMode = ccMode;
    }

    public List<String> getBcc() {
        return bcc;
    }

    public void setBcc(List<String> bcc) {
        this.bcc = bcc;
    }

    public EmailOverrideMode getBccMode() {
        return bccMode;
    }

    public void setBccMode(EmailOverrideMode bccMode) {
        this.bccMode = bccMode;
    }

    public List<String> getExtraDataKeys() {
        return extraDataKeys;
    }

    public void setExtraDataKeys(List<String> extraDataKeys) {
        this.extraDataKeys = extraDataKeys;
    }

    public FieldOptionsPersist getBodyFieldOptions() {
        return bodyFieldOptions;
    }

    public void setBodyFieldOptions(FieldOptionsPersist bodyFieldOptions) {
        this.bodyFieldOptions = bodyFieldOptions;
    }

    @Component(NotificationTemplateValuePersist.NotificationTemplateValuePersistValidator.ValidatorName)
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public static class NotificationTemplateValuePersistValidator extends BaseValidator<NotificationTemplateValuePersist> {

        public static final String ValidatorName = "NotificationTemplateValuePersistValidator";

        private final MessageSource messageSource;

        private final ValidatorFactory validatorFactory;

        protected NotificationTemplateValuePersistValidator(ConventionService conventionService, ErrorThesaurusProperties errors, MessageSource messageSource, ValidatorFactory validatorFactory) {
            super(conventionService, errors);
            this.messageSource = messageSource;
            this.validatorFactory = validatorFactory;
        }

        @Override
        protected Class<NotificationTemplateValuePersist> modelClass() {
            return NotificationTemplateValuePersist.class;
        }

        @Override
        protected List<Specification> specifications(NotificationTemplateValuePersist item) {
            return Arrays.asList(
                    this.spec()
                            .must(() -> !this.isEmpty(item.getSubjectKey()))
                            .failOn(NotificationTemplateValuePersist._subjectKey).failWith(messageSource.getMessage("Validation_Required", new Object[]{NotificationTemplateValuePersist._subjectKey}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .must(() -> !this.isEmpty(item.getSubjectText()))
                            .failOn(NotificationTemplateValuePersist._subjectText).failWith(messageSource.getMessage("Validation_Required", new Object[]{NotificationTemplateValuePersist._subjectText}, LocaleContextHolder.getLocale())),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getSubjectFieldOptions()))
                            .on(NotificationTemplateValuePersist._subjectFieldOptions)
                            .over(item.getSubjectFieldOptions())
                            .using(() -> this.validatorFactory.validator(FieldOptionsPersist.FieldOptionsPersistValidator.class)),
                    this.spec()
                            .must(() -> !this.isEmpty(item.getBodyKey()))
                            .failOn(NotificationTemplateValuePersist._bodyKey).failWith(messageSource.getMessage("Validation_Required", new Object[]{NotificationTemplateValuePersist._bodyKey}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .must(() -> !this.isEmpty(item.getBodyText()))
                            .failOn(NotificationTemplateValuePersist._bodyText).failWith(messageSource.getMessage("Validation_Required", new Object[]{NotificationTemplateValuePersist._bodyText}, LocaleContextHolder.getLocale())),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getBodyFieldOptions()))
                            .on(NotificationTemplateValuePersist._bodyFieldOptions)
                            .over(item.getBodyFieldOptions())
                            .using(() -> this.validatorFactory.validator(FieldOptionsPersist.FieldOptionsPersistValidator.class)),
                    this.spec()
                            .must(() -> !this.isNull(item.getAllowAttachments()))
                            .failOn(NotificationTemplateValuePersist._allowAttachments).failWith(messageSource.getMessage("Validation_Required", new Object[]{NotificationTemplateValuePersist._allowAttachments}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .must(() -> !this.isEmpty(item.getPriorityKey()))
                            .failOn(NotificationTemplateValuePersist._priorityKey).failWith(messageSource.getMessage("Validation_Required", new Object[]{NotificationTemplateValuePersist._priorityKey}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .must(() -> !this.isNull(item.getCcMode()))
                            .failOn(NotificationTemplateValuePersist._ccMode).failWith(messageSource.getMessage("Validation_Required", new Object[]{NotificationTemplateValuePersist._ccMode}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .must(() -> !this.isNull(item.getBccMode()))
                            .failOn(NotificationTemplateValuePersist._bccMode).failWith(messageSource.getMessage("Validation_Required", new Object[]{NotificationTemplateValuePersist._bccMode}, LocaleContextHolder.getLocale()))
            );
        }
    }
}
